import { ErrorContainer } from './error.container';
import { Store } from '@ngrx/store';
import * as fromApp from '../reducers';
import { LocalizedRouterService } from '../services/localized-router/localized-router.service';
import { DialogService } from '@hdca/fe-angular-dialog';

export const ui = 20;
export const accessibility = null;
describe('Cart 2.0', () => {
    describe('ErrorContainer', () => {
        let storeMock: jasmine.SpyObj<Store<fromApp.State>>;
        let routerMock: jasmine.SpyObj<LocalizedRouterService>;
        let dialogServiceMock: jasmine.SpyObj<DialogService>;
        let errorContainer: ErrorContainer;
        beforeEach(() => {
            storeMock = jasmine.createSpyObj<Store<fromApp.State>>('Store', {'select': { 'subscribe': () => {}}});
            routerMock = jasmine.createSpyObj<LocalizedRouterService>('LocalizedRouterService', ['go']);
            dialogServiceMock = jasmine.createSpyObj<DialogService>('DialogService', ['close', 'open']);
            errorContainer = new ErrorContainer(storeMock, routerMock, dialogServiceMock);
        });
        it('should have ModalContainer defined', () => {
            expect(errorContainer).toBeTruthy();
        });
        it('should have `dialogFooter` value as `defined`', () => {
            const fn = () => {
                dialogServiceMock.close('api-dialog');
                if (errorContainer.redirectURL) {
                    routerMock.go(errorContainer.redirectURL);
                }
            };
            const dialogFooter = [
                {
                    value: 'Go Back',
                    classes: 'hdca-button--primary',
                    onClick: fn
                }
            ];
            errorContainer.dialogFooter[0].onClick = fn;
            expect(errorContainer.dialogFooter).toBeTruthy();
            expect(errorContainer.dialogFooter).toEqual(dialogFooter);
        });
        it('should have `requestURL` value as `empty`', () => {
            expect(errorContainer.requestURL).toEqual('');
        });
        it('should have `redirectURL` value as `empty`', () => {
            expect(errorContainer.redirectURL).toEqual('');
        });
        it('should have ngOnInit method created', () => {
            expect(errorContainer.ngOnInit).toBeTruthy();
        });
        it('should have ngOnDestroy method created', () => {
            expect(errorContainer.ngOnDestroy).toBeTruthy();
        });
    });
});
