import { NgModule, Optional, SkipSelf, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { StoreModule } from '@ngrx/store';
import { EffectsModule } from '@ngrx/effects';
import { HDCADialogModule } from '@hdca/fe-angular-dialog';
import { HttpClientModule, HTTP_INTERCEPTORS } from '@angular/common/http';
import { StoreDevtoolsModule } from '@ngrx/store-devtools';
import { FEAHttpInterceptor } from './services/http-interceptor';
import { TokenRefreshService } from './services/token-refresh/token-refresh.service';

import { ModalEffects } from './effects/modal.effects';
import * as fromApp from './reducers';

// modules
 import { HDCAGlobalFooterModule } from './modules/global-footer/global-footer.module';
// import { HDCAGlobalNotificationModule } from './modules/global-notification/global-notification.module';
// import { HDCAEventTransceiverModule } from './modules/event-transceiver/event-transceiver.module';
// import { HDCADialogModule } from '@hdca/fe-angular-dialog';
// import { HDCAGlobalCartModule } from './modules/global-cart/global-cart.module';
// import { HDCAGlobalHeaderModule } from './modules/global-header/global-header.module';

// components and containers
// import { ErrorContainer } from './containers/error.container';
import { ErrorComponent } from './components/error.component';

// services
import { LocalizedRouterService } from './services/localized-router/localized-router.service';
import { OriginService } from './services/origin/origin.service';
import { GeoLocatorService } from './services/geo-locator/geo-locator.service';
import { NavNodesResolver } from './resolvers/nav-nodes.resolver';
import { DataFetcher } from './services/data-fetcher/data-fetcher.service';
import { DataSender } from './services/data-sender/data-sender.service';
import { BingMapService } from './services/bing-maps/bing-maps.service';
import { BingMapFsaService } from './services/bing-maps/bing-maps.fsaservice';
import { MyAccountLoginPortal } from './services/data-sender/my-account-login.service';

import { domain, production, subDomain, hash } from './tokens';

interface IEnvironment {
  domain: string;
  appName: string;
  envName: string;
  production: boolean;
  versions: any;
}

interface Iversion {
  dirty: boolean;
  raw: string;
  hash: string;
  distance: any;
  tag: any;
  semver: any;
  suffix: string;
  semverString: string;
  version: string;
  branch: string;
}

@NgModule({
  imports: [
    CommonModule,
    HttpClientModule,
    HDCADialogModule.forRoot(),
    // HDCAEventTransceiverModule.forRoot(),
     HDCAGlobalFooterModule.forRoot(),
    // HDCAGlobalNotificationModule.forRoot(),
    // HDCADialogModule.forRoot(),
    // HDCAGlobalCartModule.forRoot(),
    // HDCAGlobalCartModule.forRoot(),
    // StoreModule.forRoot(fromApp.reducers, { metaReducers: fromApp.metaReducers }),
    // production
    //   ? StoreDevtoolsModule.instrument({
    //     maxAge: 5
    //   })
    //   : [],
    // EffectsModule.forRoot([
    //   ModalEffects
    // ])
  ],
  declarations: [
    // ErrorContainer
    ErrorComponent
  ],
  exports: [
    // HDCAEventTransceiverModule,
    // HDCAGlobalHeaderModule,
     HDCAGlobalFooterModule,
    // HDCAGlobalNotificationModule,
    // HDCAGlobalCartModule
    ErrorComponent
  ]
})
export class HDCAFeAngularCore {
  constructor(
    @Optional()
    @SkipSelf()
    parentModule: HDCAFeAngularCore
  ) {
    if (parentModule) {
      throw new Error('CoreModule is already loaded. Import only in AppModule');
    }
  }

  static forRoot(env: IEnvironment, versionInput: Iversion, subDomainInput: string): ModuleWithProviders {
    return {
      ngModule: HDCAFeAngularCore,
      providers: [
        // {
        //   provide: HTTP_INTERCEPTORS,
        //   useClass: FEAHttpInterceptor,
        //   multi: true
        // },
        {
          provide: domain,
          useValue: env.domain
        },
        {
          provide: production,
          useValue: env.production
        },
        {
          provide: hash,
          useValue: versionInput.hash
        },
        {
          provide: subDomain,
          useValue: subDomainInput
        },
        LocalizedRouterService,
        OriginService,
        GeoLocatorService,
        TokenRefreshService,
        // NavNodesResolver,
        DataFetcher,
        DataSender,
        BingMapService,
        BingMapFsaService,
        MyAccountLoginPortal
      ]
    };
  }
}
