export interface ProductLocalizedData {
  flip?: boolean;
  aisleBay?: AisleBay;
  id: string;
  badges?: Badge[];
  bodfs: boolean;
  bopis: boolean;
  boss: boolean;
  shipToHome: boolean;
  buyable: boolean;
  cacheable: boolean;
  timestamp?: Date;
  fulfillmentMessages?: FulfillmentMessages;
  installEligible: boolean;
  installService: InstallService;
  isBopisOutOfAreaEnabled: boolean;
  onlineStock: OnlineStock;
  optimizedPrice: OptimizedPrice;
  potentialPromotions?: PotentialPromotion[];
  promotionMessages: PromotionMessages;
  storeStock: OnlineStock;
}

export interface Badge {
  code: string;
  name: string;
  fromDate?: string;
  toDate?: string;
}

export interface PotentialPromotion {
  code: string;
  promotionAccordion?: string;
  promotionMessageForPIP?: string;
  promotionType: string;
}

export interface PromotionMessages {
  accordionMessages?: string[];
  pipMessage?: string;
  plpMessage?: string;
  localizedPipMessage?: string;
}

interface OptimizedPrice {
  availabilityMsg: string;
  percentSaving?: string;
  displayPrice?: DisplayPrice;
  comparablePrice?: ComparablePrice;
  endDate: string;
  productId: string;
  productStatus?: string;
  regprice: RegPrice;
  savingsAmount?: SavingsAmount;
  storeId: string;
  wasprice?: WasPrice;
  promoprice?: PromoPrice;
}

export interface DisplayPrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
  unitOfMeasure?: string;
}

interface RegPrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

interface PromoPrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

interface SavingsAmount {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

interface ComparablePrice {
  comparableUnitofMeasure: string;
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

interface WasPrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

interface OnlineStock {
  stockLevel: number;
  stockLevelStatus: string;
}

export interface AvailableMessages {
  bopis: boolean;
  boss: boolean;
  bodfs: boolean;
  shipToHome: boolean;
  availabilityMessage: boolean;
  findInStore: boolean;
  addToCart: boolean;
}

export interface FulfillmentMessages {
  availableMessages: AvailableMessages;
  addToCart?: string;
  availabilityMessage?: string;
  bopis?: Bopis;
  boss?: Boss;
  bodfs?: boolean;
  findInStore?: FindInStore;
  shipToHome?: ShipToHome;
}

interface ShipToHome {
  enabled: boolean;
  estCostDisplay: boolean;
  estTimeDisplay: boolean;
  flatRateDisplay: boolean;
  freeShipping: boolean;
  messageTranslKey: string;
}

export interface FindInStore {
  messageTranslKey: string;
  pickUpStore: boolean;
  showAisleBay: boolean;
  variation: number;
  state?: string;
  qty?: number;
  storeStockLevel: number;
  storeLinkTextTranslKey?: string;
}

export interface Bopis {
  enabled: boolean;
  messageTranslKey: string;
  pickUpStore: boolean;
  pickUpTime: boolean;
  storeLinkTextTranslKey: string;
  storeStockLevel?: number;
}

interface Boss {
  enabled: boolean;
  pickUpStore: boolean;
  pickUpTime: boolean;
  storeLinkTextTranslKey: string;
  messageTranslKey?: string;
}

export interface AisleBay {
  aisleLocation?: string;
  bayLocation?: string;
  productCode: string;
  storeDisplayName: string;
  storeId: string;
}

export interface PickUpStoreCookie {
  storeID: string;
  displayName: string;
}

// Raw response from Hybris localized product.
interface InstallService {
  code: string;
  description: string;
  name: string;
  selected: boolean;
  cost: RegPrice;
}

// Digested object for PIP page InstallService component. Cherrypicked required attributes from ApplianceAvailabilityData and ProductLocalized.InstallService.
export interface InstallServiceData {
  code: string;
  eligible: boolean;
  selected: boolean;
  cost: RegPrice;
  available: boolean;
  tripCost: number;
  tripCostFormatted: string;
  errorMessage: string;
}
