import { Breadcrumb } from '@hdca/fe-angular-shared';

interface ProductUnlocalizedData {
  gallery?: string[];
  alternateImages?: AlternateImage[];
  articleType: string;
  availableForPickup: boolean;
  availableRelationships?: String[];
  averageRating: number;
  badges: Badge[];
  bodfs: boolean;
  bopis: boolean;
  boss: boolean;
  buyable: boolean;
  categories?: Badge[];
  classifications?: Classification[];
  code: string;
  countryOfOrigin: string;
  deliveryTime: string;
  deliveryUnit: string;
  depotDirect: boolean;
  description: string;
  documents?: Document[];
  ecoFeeEligible: boolean;
  freeShipping: number;
  images?: AlternateImage[];
  installationEligible: boolean;
  isBopisOutOfAreaEnabled: boolean;
  itemDepth: number;
  itemHeight: number;
  itemWeight: number;
  itemWidth: number;
  legacyId: string;
  manufacturer: string;
  messageType: number;
  modelNumber: string;
  modifiedtime: string;
  name: string;
  numberOfQuestions: number;
  numberOfReviews: number;
  price?: Price;
  productFeatures: ProductFeature[];
  productReferences?: ProductReference[];
  promotionMessages: PromotionMessages;
  purchasable: boolean;
  sellableIntent: string;
  shipToStoreTime: string;
  shipToStoreUnit: string;
  soArticle?: string;
  stock: Stock;
  storeInventoryLookup: boolean;
  summary: string;
  unitOfMeasure: string;
  url?: string;
  urls: {
    en: string;
    fr: string;
  };  videoTags?: string[];
  warranty: string;
  inlineContent: string[];
  inlineSubstitute: string;
  breadCrumbs?: Breadcrumb[];
  variantOptionsSorted?: Variant[];
  flatrateFee?: FlatrateFee;
  relatedLinks?: Bloomreach;
}

interface Bloomreach {
  brIuid: string;
  brRelatedRid: string;
  relatedSearches?: RelatedSearch[];
  relatedProducts: RelatedProducts[];
}

interface RelatedSearch {
  anchor: string;
  url: string;
}

interface RelatedProducts {
  productCode: string;
  productUrl: string;
  imageUrl: string;
  title: string;
  description: string;
  index?: number;
}

interface FlatrateFee {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface Variant {
  name: string;
  hasImage: boolean;
  selectedVariantOptionQualifier: VariantOptionQualifier;
  variantOptionQualifiers: VariantOptionQualifier[];
}

export interface VariantOptionQualifier {
  selected: boolean;
  available: boolean;
  value: string;
  url?: string;
  image?: VariantOptionQualifierImage;
  code?: string;
  position?: number;
  variantCategoryCode?: string;
  variantValueCategoryCode?: string;
}

export interface VariantOptionQualifierImage {
  altText: string;
  url?: string;
}
interface Stock {
  stockLevel: number;
  stockLevelStatus: string;
}

interface PromotionMessages {
  pipMessage?: string;
  plpMessage?: string;
}

 interface ProductFeature {
  code: string;
  comparable: boolean;
  featureValues: FeatureValue[];
  name: string;
  range: boolean;
}

 interface ProductReference {
  referenceType: string;
  target: ProductTarget;
  preselected: boolean;
}

 interface ProductTarget {
  code: string;
  name: string;
  url: string;
}

interface Price {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

 interface Document {
  URL: string;
  mime: string;
  name: string;
}

 interface Classification {
  code: string;
  features: Feature[];
  name: string;
}

 interface Feature {
  code: string;
  comparable: boolean;
  featureUnit?: FeatureUnit;
  featureValues: FeatureValue[];
  name: string;
  range: boolean;
}

 interface FeatureValue {
  value: string;
}

 interface FeatureUnit {
  name: string;
  symbol: string;
  unitType: string;
}

 interface Badge {
  code: string;
  name: string;
}

 interface AlternateImage {
  altText: string;
  format: string;
  imageType: string;
  url: string;
  isVideoThumbnail?: boolean;
  videoId?: number;
}
