export interface ApplianceOrderGroup {
    applianceOrderGroups: ApplianceOrderGroupItem[];
}

export interface ApplianceOrderGroupItem {
    entries: ApplianceEntry[];
    totalPriceWithTax: ApplianceDeliveryCost;
    ecoFee: ApplianceDeliveryCost;
    taxForGST: ApplianceDeliveryCost;
    taxForHST: ApplianceDeliveryCost;
    taxForPST: ApplianceDeliveryCost;
}

export interface ApplianceEntry {
    applianceEntrySubtotal: ApplianceDeliveryCost;
    applianceEstimatedDeliveryDate: string;
    applianceVendorNumber: string;
    basePrice: ApplianceDeliveryCost;
    basePriceTotal: ApplianceDeliveryCost;
    basePriceValue: ApplianceDeliveryCost;
    displayPrice: ApplianceDeliveryCost;
    ecoFee: ApplianceDeliveryCost;
    entryNumber: number;
    isApplianceOrAddon: boolean;
    isFreeGift: boolean;
    onlineStock: OnlineStock;
    proDirectQualifier: string;
    product: Product;
    promotionalSavings: ApplianceDeliveryCost;
    quantity: number;
    requiresProductAddons: boolean;
    shippingFee: ApplianceDeliveryCost;
    totalPrice: ApplianceDeliveryCost;
    totalStockQuantity: number;
    updateable: boolean;
}

 interface Product {
    articleType: string;
    availableForPickup: boolean;
    boss: boolean;
    buyable: boolean;
    categories: Category[];
    code: string;
    installServices: InstallService[];
    partsAndServices?: PartsAndServices;
    deliveryUnit: string;
    depotDirect: boolean;
    description: string;
    ecoFeeEligible: boolean;
    freeShipping: number;
    images: Image[];
    manufacturer: string;
    modelNumber: string;
    name: string;
    optimizedPrice: OptimizedPrice;
    price: ApplianceDeliveryCost;
    purchasable: boolean;
    sellableIntent: string;
    shipToHome: boolean;
    shipToStoreUnit: string;
    stock: OnlineStock;
    unitOfMeasure: string;
    url: string;
}

export interface InstallService {
    code: string;
    cost?: ApplianceDeliveryCost;
    price?: string;
    description: string;
    name: string;
    selected: boolean;
    type: string;
    source: string;
    isHaulAwayIS?: boolean;
}

export interface PartsAndServices {
    hasParts: boolean;
    parts: AddOn[];
    services: InstallService[];
    warranty: AddOn[];
}

export interface AddOn {
    code: string;
    name: string;
    description: string;
    price: string;
    source: string;
    selected: boolean;
    type?: string;
    cost?: ApplianceDeliveryCost;
    isHaulAwayIS?: boolean;
}

interface OptimizedPrice {
    availabilityMsg: string;
    displayPrice?: ApplianceDeliveryCost;
    endDate: string;
    productId: string;
    regprice: ApplianceDeliveryCost;
    savingsAmount: ApplianceDeliveryCost;
    storeId: string;
}

interface Image {
    altText: string;
    format: string;
    imageType: string;
    url: string;
}

interface Category {
    code: string;
    url: string;
}

interface OnlineStock {
    stockLevel: number;
    stockLevelStatus: string;
}

interface ApplianceDeliveryCost {
    currencyIso: string;
    formattedValue: string;
    priceType: string;
    value: number;
}
