import {PickupOrderGroupItem} from './pick-up.interface';
import {ApplianceOrderGroupItem} from './appliance-delivery.interface';
import {Alert} from './alert.interface';
import { TemplateRef } from '@angular/core';

export interface ProductList {
    productList: ProductItem[];
}

export interface ProductItem {
    type: string;
    applianceDeliveryCost: BasePrice;
    installationTripCost: BasePrice;
    installServiceAvailable: boolean;
    appliancePostalCode: string;
    applianceRequestedShipDate: string;
    calculated: boolean;
    code: string;
    deliveryCost: BasePrice;
    appliedOrderPromotions: AppliedOrderPromotion[];
    appliedProductPromotions: AppliedProductPromotion[];
    applianceOrderGroups: ApplianceOrderGroupItem[];
    deliveryOrderGroups: DeliveryOrderGroup[];
    pickupOrderGroups: PickupOrderGroupItem[];
    entries: Entry[];
    grandOrderTotal: BasePrice;
    guid: string;
    isCartBuyable: boolean;
    isRuralPostalCode: boolean;
    net: boolean;
    orderDiscounts: BasePrice;
    pickupItemsQuantity: number;
    productDiscounts: BasePrice;
    sisterStoreLookup: boolean;
    site: string;
    store: string;
    subTotal: BasePrice;
    subTotalAppliance: BasePrice;
    subTotalMerchandise: BasePrice;
    taxForGST: BasePrice;
    taxForHST: BasePrice;
    taxForPST: BasePrice;
    totalDiscounts: BasePrice;
    totalEcoFees: BasePrice;
    totalItems: number;
    totalPrice: BasePrice;
    totalPriceWithTax: BasePrice;
    totalSavings: BasePrice;
    totalTax: BasePrice;
    totalDeliveryCost: BasePrice;
    user: User;
    isShippingFree: boolean;
    isCheckoutButtonEnabled: boolean;
    isExpressCheckout: boolean;
    totalItemsQuantity: number;
    applianceItemsQuantity: number;
    deliveryItemsQuantity: number;
    bopisItemsQuantity: number;
    bossItemsQuantity: number;
    bodfsItemsQuantity: number;
    ui: UIState;
    defaultPostalCode: string;
    isPromotionUpdated: boolean;
}

export interface DeliveryMode {
    code: string;
    name: string;
}

export interface AppliedOrderPromotion {
    description: string;
    promotion: AppliedOrderPromotionItem;
}

export interface AppliedOrderPromotionItem {
    code: string;
    description: string;
    endDate: string;
    promotionAccordion: string;
    promotionMessageForPIP: string;
    promotionType: string;
}

export interface UIState {
    alerts: Alert[];
    shipToYou: ShipToYouUIState;
    productAlerts: ProductAlert[];
    appliance: ApplianceUIState;
    bodfs: BodfsUIState;
    paypal: PaypalUIState;
    animateEntryNumber: number;
}

export interface PaypalUIState {
    loading: boolean;
}

export interface BodfsUIState {
    postalCodeBoxLoading: boolean;
    postalCodeBoxEditMode: boolean;
    deliveryDateLoading: boolean;
    postalCodeBoxClearText: boolean;
    alerts: Alert[];
}

export interface ApplianceUIState {
    postalCodeBoxLoading: boolean;
    postalCodeBoxEditMode: boolean;
    alerts: Alert[];
}

export interface ProductAlert {
    product: string;
    alert: Alert;
}

export interface ShipToYouUIState {
    postalCodeBoxLoading: boolean;
    postalCodeBoxEditMode: boolean;
    alerts: Alert[];
}

export interface User {
    uid: string;
}

export interface DeliveryOrderGroup {
    entries: Entry[];
    totalPriceWithTax: BasePrice;
    ecoFee: BasePrice;
    shippingFee: BasePrice;
    taxForGST: BasePrice;
    taxForHST: BasePrice;
    taxForPST: BasePrice;
    deliveryOrderType: string;
    deliveryPostalCode: string;
    earliestDeliveryDate: number;
    bodfsRequestedDeliveryDate: string;
    deliveryAddress: string;
    hasRemovedItem?: boolean;
    personalizedETADestination?: any;
    deliveryPointOfService?: any;
}

export interface Entry {
    applianceVendorNumber: string;
    articleLevelMessage?: string;
    basePrice: BasePrice;
    basePriceTotal: BasePrice;
    basePriceValue: BasePrice;
    displayPrice: BasePrice;
    wasPrice: BasePrice;
    deliveryMode?: DeliveryMode;
    entryNumber: number;
    isApplianceOrAddon: boolean;
    isFreeGift: boolean;
    onlineStock: OnlineStock;
    proDirectQualifier: string;
    deliveryPointOfService?: any;
    parentEntryNumber?: string;
    product: Product;
    fulfillmentOptions: FulfillmentOptions;
    promotionalSavings: BasePrice;
    quantity: number;
    requiresProductAddons: boolean;
    shippingFee: BasePrice;
    totalPrice: BasePrice;
    totalStockQuantity: number;
    updateable: boolean;
}

export interface FulfillmentOptions {
    shipToYou: ShipToYouFulfillment;
    pickupInStore: PickUpFulfillment;
    scheduledDelivery: ScheduledDeliveryFulfillment;
}

export interface FulfillmentOption {
    selected: boolean;
    displayOrder: number;
}

export interface ScheduledDeliveryFulfillment extends FulfillmentOption {
    deliveryDate: string;
    curbsideDelivery?: boolean;
}

export interface ShipToYouFulfillment extends FulfillmentOption {
    freeShipping: boolean;
    shippingFee: ShippingFee;
    deliveryTime: string;
    deliveryUnit: string;
    curbsideDelivery?: boolean;
}

export interface ShippingFee {
    currencyIso: string;
    formattedValue: string;
    priceType: string;
    value: number;
}

export interface PickUpFulfillment extends FulfillmentOption {
    changeStore: boolean;
    storeStock?: StoreStock;
    store: Store;
    shipToStoreTime?: string;
    shipToStoreUnit?: string;
}

export interface FulfillmentOptionUIListItem {
    order: number;
    template: TemplateRef<any>;
}

export interface StoreStock {
    stockLevel: number;
    stockLevelStatus: string;
}

export interface Store {
    name: number;
    displayName: string;
}

export interface Product {
    articleType: string;
    availableForPickup: boolean;
    bopis: boolean;
    boss: boolean;
    buyable: boolean;
    categories: Category[];
    code: string;
    deliveryTime: string;
    deliveryUnit: string;
    depotDirect: boolean;
    description: string;
    ecoFeeEligible: boolean;
    freeShipping: number;
    manufacturer: string;
    modelNumber: string;
    name: string;
    optimizedPrice: OptimizedPrice;
    price: BasePrice;
    purchasable: boolean;
    sellableIntent: string;
    shipToHome: boolean;
    shipToStoreTime: string;
    shipToStoreUnit: string;
    stock: OnlineStock;
    unitOfMeasure: string;
    url: string;
    images: Images[];
}

export interface OptimizedPrice {
    availabilityMsg: string;
    displayPrice?: BasePrice;
    wasprice: BasePrice;
    endDate: string;
    productId: string;
    productStatus: string;
    regprice: BasePrice;
    savingsAmount: BasePrice;
    storeId: string;
}

export interface Category {
    code: string;
    url: string;
}

export interface Images {
    altText: string;
    format: string;
    imageType: string;
    url: string;
}

export interface OnlineStock {
    stockLevel: number;
    stockLevelStatus: string;
}

interface BasePrice {
    currencyIso: string;
    formattedValue: string;
    priceType: string;
    value: number;
}

export interface AppliedProductPromotion {
    consumedEntries: ConsumedEntry[];
    description: string;
    promotion: Promotion;
}

interface Promotion {
    code: string;
    description: string;
    endDate: string;
    promotionAccordion: string;
    promotionMessageForPIP: string;
    promotionType: string;
}

interface ConsumedEntry {
    adjustedUnitPrice: number;
    orderEntryNumber: number;
    quantity: number;
}

export interface InstallationServiceState {
    payload: boolean;
    index: number;
}
