import {FulfillmentOptions} from './cart.interface';

export interface PickupOrderGroup {
    pickupOrderGroups: PickupOrderGroupItem[];
}

export interface PickupOrderGroupItem {
    entries: PickupEntry[];
    quantity: number;
    totalPriceWithTax: BasePrice;
    deliveryPointOfService: DeliveryPointOfService;
    ecoFee: BasePrice;
    pickupOrderType: string;
    taxForGST: BasePrice;
    taxForHST: BasePrice;
    taxForPST: BasePrice;
    bopisItemsQuantity: number;
    personalizedETADestination?: any;
}

export interface PickupEntry {
    applianceVendorNumber: string;
    basePrice: BasePrice;
    basePriceTotal: BasePrice;
    basePriceValue: BasePrice;
    deliveryPointOfService?: DeliveryPointOfService;
    deliveryStoreStock?: DeliveryStoreStock;
    displayPrice: BasePrice;
    entryNumber: number;
    fulfillmentOptions: FulfillmentOptions;
    isApplianceOrAddon: boolean;
    isFreeGift: boolean;
    onlineStock: DeliveryStoreStock;
    proDirectQualifier: string;
    product: PickupProduct;
    productStatus?: string;
    promotionalSavings: BasePrice;
    quantity: number;
    requiresProductAddons: boolean;
    shippingFee: BasePrice;
    totalPrice: BasePrice;
    totalStockQuantity: number;
    updateable: boolean;
}

export interface PickupProduct {
    articleType: string;
    availableForPickup: boolean;
    bopis: boolean;
    boss: boolean;
    buyable: boolean;
    categories: Category[];
    code: string;
    deliveryUnit: string;
    depotDirect: boolean;
    description: string;
    ecoFeeEligible: boolean;
    freeShipping: number;
    manufacturer: string;
    modelNumber: string;
    name: string;
    optimizedPrice: OptimizedPrice;
    price: BasePrice;
    purchasable: boolean;
    sellableIntent: string;
    shipToHome: boolean;
    shipToStoreUnit: string;
    stock: DeliveryStoreStock;
    unitOfMeasure: string;
    url: string;
}

interface OptimizedPrice {
    availabilityMsg: string;
    displayPrice?: BasePrice;
    endDate: string;
    productId: string;
    productStatus: string;
    regprice: BasePrice;
    savingsAmount: BasePrice;
    storeId: string;
}

interface Category {
    code: string;
    url: string;
}

interface DeliveryStoreStock {
    stockLevel: number;
    stockLevelStatus: string;
}

export interface DeliveryPointOfService {
    address: Address;
    displayName: string;
    name: string;
    openingHours: OpeningHours;
    geoPoint: GeoPoint;
}

export interface Address {
    formattedAddress: string;
    phone: string;
}

interface BasePrice {
    currencyIso: string;
    formattedValue: string;
    priceType: string;
    value: number;
}

interface OpeningHours {
    code: string;
    weekDayOpeningList: WeekDayOpeningList[];
}

interface WeekDayOpeningList {
    closingTime: ClosingTime;
}

interface ClosingTime {
    formattedHour: string;
}

interface GeoPoint {
    latitude: number;
    longitude: number;
}
