import { Breadcrumb } from '@hdca/fe-angular-shared';
import * as constants from '../constants/adobe-analytics.constants';

export interface BasicPayload {
  event: EventBlock;
  user?: UserBlock;
}

export interface DataWrapperBlock {
  cartData?: any;
  nullSearchData?: any;
  certona?: any;
  redirect?: any;
  productUnlocalized?: any;
  productLocalized?: any;
  fbtData?: any;
}

export interface PageAndModalViewPayload extends BasicPayload {
  pageDetail: PageBlock;
  productOverview?: {
    productId: string;
    attributes: ProductBasicAttributeBlock | ProductBasicAttributeLocalizedBlock;
    superSku?: {
      eligible: boolean,
      attributeCount?: number;
      attribute?: Array<ProductSuperSkuAttributeBlock>
    };
    fbt?: {
      item: Array<ProductFBTBlock>
    };
    relatedSearches?: Array<any>;
    relatedProducts?: Array<any>
  };
  certona?: {
    recommendedProductList?: Array<ProductBasicAttributeBlock>,
    accessoriesProductList?: Array<ProductBasicAttributeBlock>,
    collectionProductList?: Array<ProductBasicAttributeBlock>,
    trendingProductList?: Array<ProductBasicAttributeBlock>
  };
  nullSearchPage?: {
    carousels?: Array<NullSearchCarouselBlock>
  };
  user?: UserBlock;
  cart?: CartBlock;
}

export interface NullSearchCarouselBlock {
  relatedQuery: string;
  products: Array<ProductBasicAttributeBlock>;
}

export interface SuperSkuSwitchPayload extends BasicPayload {
  superSkuSwitchData: SuperSkuSwitchBlock;
}

export interface CarouselProductClickPayload extends BasicPayload {
  productId: string;
  contextProductId: string;
  slotLocation: number;
  productInteractionLocation: constants.productInteractionLocationTypes;
  productInteractionPageType: constants.pageTypes;
  productInteractionLocationTitle: string;
  productInteractionLocationOrder?: number;
}

export interface PETAPayload extends BasicPayload {
  peta?: {
    deliveryDays: {
      sth: string;
      boss: string;
    }
  };
}

export interface ElementClickPayload extends BasicPayload {
  elemenntClickBlock: {
    elementType: constants.clickElementTypes;
    elementSubType?: constants.clickElementSubTypes;
    elementValue: constants.clickElementValues;
  };
}

export interface SiteErrorPayload extends BasicPayload {
  error: SiteErrorBlock;
}

export interface AddToCartPayload extends BasicPayload {
  items: AddToCartInfoBlock[];
}

// Building blocks for each payload
export interface EventBlock {
  type: constants.eventTypes;
  timestamp?: number;
  page?: constants.pageTypes;
  language?: constants.languageTypes;
}

export interface PageBlock {
  pageName: string;
  sysEnv: string;
  pageContentType: string;
  pageSiteSection?: string;
  breadcrumbs?: Breadcrumb[];
  pageCategory?: string;
  pageCategory1?: string;
  pageCategory2?: string;
  pageCategory3?: string;
  pageCategory4?: string;
  pageCategory5?: string;
  search: SearchRedirectBlock;
}

export interface SearchRedirectBlock {
  redirect: boolean;
  keyword: string;
}

interface ProductBasicAttributeBlock {
  code?: string;
  eligible?: string;
  freeShipping?: number;
  rating?: number;
  reviews?: number;
  questions?: number;
  storeShippingAvailability?: string;
}

interface ProductBasicAttributeLocalizedBlock extends ProductBasicAttributeBlock {
  localizedAssortment?: string;
  onlineStockAvailability?: string;
  storeStockAvailability?: string;
  regPrice?: string;
  wasPrice?: string;
}

interface ProductSuperSkuAttributeBlock {
  attributeId: string;
  attributeName: string;
  valueIds: string[];
  valueNames: string[];
  activeValueId: string;
  activeValueName: string;
}

interface ProductFBTBlock {
  slotLocation: number;
  productId: string;
  sellPrice: number;
}

export interface UserBlock {
  guestStatus: string;
  profile: {
    profileInfo: {
      email: string;
      language: constants.languageTypes;
      profileId: string;
      type: string;
    }
  };
}

export interface ProductInteractionBlock {
  // Common
  slotLocation: number;
  productInteractionLocation: constants.productInteractionLocationTypes;
}

export interface ProductClickBlock extends ProductInteractionBlock {
  productId: string;
  contextProductId: string;
  productInteractionLocationTitle: string;
  productInteractionPageType: constants.pageTypes;
  productInteractionLocationOrder?: number;
}

export interface AddToCartInfoBlock extends ProductInteractionBlock {
  product: ProductBasicAttributeBlock;
  itemQuantity: string | number;
  displayPrice?: string | number;
  isCartSwitch: boolean;
  displayShippingMethod: constants.shippingMethodTypes;
  searchTermTitle?: string;
  carouselOrder?: number;
}

export interface SiteErrorBlock {
  errorType: constants.siteErrorTypes;
  userInformed: boolean;
  errorCode: string;
  errorValue: string;
  target: string; // string identifier - url, element name
  errorPayload?: any; // any payload associated with the error
}

export interface CartBlock {
  active: boolean;
  id: string;
  item: Array<CartItemDataBlock>;
  attributes: {
    totalUnits: number;
    shipping: {
      mixed: string[];
    }
    productcount: number;
    deliveryCost: number;
  };
}

interface CartItemDataBlock {
  productId: string;
  price?: {
    baseprice: number;
    discountPrice: number;
    voucherCode: number;
    discount: number;
    shippingMethod: string;
  };
  quantity: number;
  itemTotal: number;
  deliveryDays: string;
}

export interface InputErrorEventBlock {
  element: string;
  error: {
    code: string;
    message: string;
    needsTranslation: boolean;
  };
}

export interface SuperSkuSwitchBlock {
  productId: string;
  attributeName: string;
  attributeId: string;
  targetProductId: string;
  targetValueId: string;
  targetValueName: string;
}
