import * as EventTransceiverActions from '../actions';
import * as AdobeAnalyticsStore from './adobe-analytics.reducer';
import * as WebPerformanceStore from './web-performance.reducer';
import * as WhitelistStore from '../reducers/whitelist.reducer';
import * as EventTrackerStore from './event-tracker.reducer';
import { Whitelist } from '../models/event-transceiver.interface';

export interface EventTransceiverState {
  whitelist: Whitelist;
  adobeAnalytics: AdobeAnalyticsStore.AdobeAnalyticsState;
  webPerformance: WebPerformanceStore.WebPerformanceState;
  eventTracker: EventTrackerStore.EventTrackerState;
}

export const initialState: EventTransceiverState = {
  whitelist: WhitelistStore.initialState,
  adobeAnalytics: AdobeAnalyticsStore.initialState,
  webPerformance: WebPerformanceStore.initialState,
  eventTracker: EventTrackerStore.initialState
};

export function eventTransceiverReducer(
  state = initialState,
  action: EventTransceiverActions.Actions
): EventTransceiverState {
  switch (action.type) {
    case EventTransceiverActions.EventTransceiverActions.EventTransceiverSetWhitelistEvent:
    return { ...state, whitelist: WhitelistStore.reducer(state.whitelist, action) };
    case EventTransceiverActions.AdobeAnalyticsActions.AdobeAnalyticsDTMReadyEvent:
      return { ...state, adobeAnalytics: AdobeAnalyticsStore.reducer(state.adobeAnalytics, action) };
    case EventTransceiverActions.WebPerformanceActions.WebPerformanceUpdateRecord:
      return { ...state, webPerformance: WebPerformanceStore.reducer(state.webPerformance, action) };
    case EventTransceiverActions.WebPerformanceActions.WebPerformanceUpdateHttpRecord:
      return { ...state, webPerformance: WebPerformanceStore.reducer(state.webPerformance, action) };
    case EventTransceiverActions.EventTrackerActions.EventTrackerUpdateLog:
      return { ...state, eventTracker: EventTrackerStore.reducer(state.eventTracker, action) };
    default: {
      return state;
    }
  }
}
