import * as AdobeAnalyticsConstants from '../constants/adobe-analytics.constants';
import * as AdobeAnalyticsModels from '../models/adobe-analytics-data-layer-models.interface';
import { AdobeAnalyticsService } from './adobe-analytics.service';
import {Store} from '@ngrx/store';
import * as globalReducer from '@core/reducers';

const mockEventBlock: AdobeAnalyticsModels.EventBlock = {
  type: AdobeAnalyticsConstants.eventTypes.pageLoad,
  timestamp: undefined,
  page: AdobeAnalyticsConstants.pageTypes.pip,
  language: AdobeAnalyticsConstants.languageTypes.en
};

const mockPageDetailDDO: AdobeAnalyticsModels.PageBlock = {
  pageName: 'Vivere 8 ft. Combo Steel Stand with Double Hammock',
  pageContentType: AdobeAnalyticsConstants.pageTypes.pip,
  sysEnv: AdobeAnalyticsConstants.systemEnvironments.desktop,
  pageSiteSection: AdobeAnalyticsConstants.pageTypes.pip,
  breadcrumbs: [{
    'title': 'Home',
    'url': '/en/home.html'
  }, {
    'title': '1000755816',
    'url': '/en/home/p.dewalt-dcd796d2-20v-max-xr-li-ion-cordless-brushless-compact-hammer-drill-kit.1000755816.html'
  }],
  'pageCategory': 'Home',
  'pageCategory1': '1000755816',
  'pageCategory2': '',
  'pageCategory3': '',
  'pageCategory4': '',
  'pageCategory5': '',
  'search' : {
    redirect: false,
    keyword: ''
  }
};

const mockCartBlock: AdobeAnalyticsModels.CartBlock = {
  'active': true,
  'id': '7aeac412-9c7e-4063-97cd-31c3e9a428ea',
  'item': [{
    'productId': '1000848230',
    'price': {
      'baseprice': 329,
      'discountPrice': 329,
      'voucherCode': 6,
      'discount': 1,
      'shippingMethod': undefined,
    },
    'quantity': 2,
    'itemTotal': 658,
    'deliveryDays': '1 - 3',
  }],
  'attributes': {
    'totalUnits': 4,
    'shipping': {
      'mixed': ['BOSS', 'APPL', 'SHIPTOHOME', 'BOPIS', 'BODFS'],
    },
    'productcount': 1,
    'deliveryCost': 2,
  }
};

const mockPageOverviewDDO = {
  'productId': '',
  'attributes': {
      'eligible': 'BOPIS:INSO',
      'freeShipping': 1,
      'rating': 3,
      'reviews': 2,
      'questions': 5,
      'storeShippingAvailability': 'outOfStock',
      'regPrice': '$58.16',
      'wasPrice': '',
      'storeStockAvailability': 'inStock',
      'onlineStockAvailability': 'inStock',
      'localizedAssortment': 'Sold In-Store Only',
    },
    'superSku': {
      'eligible': true,
      'attributeCount': 2,
      'attributes': [{
        'activeValueId': 'vvc-cls_l3-blinds-shades-attribute_id_ColourFamily-yellow',
        'activeValueName': 'Yellow',
        'attributeId': 'vc-cls_l3-blinds-shades-attribute_id_ColourFamily',
        'attributeName': 'Color',
        'valueIds': ['vvc-cls_l3-blinds-shades-attribute_id_ColourFamily-green', 'vvc-cls_l3-blinds-shades-attribute_id_ColourFamily-yellow'],
        'valueNames': ['vc-cls_l3-blinds-shades-attribute_id_ColourFamily', 'vc-cls_l3-blinds-shades-attribute_id_ColourFamily'],
      }, {
        'activeValueId': 'vvc-cls_l3-blinds-shades-attribute_id_SizeFamily-8_9',
        'activeValueName': '8X9',
        'attributeId': 'vc-cls_l3-blinds-shades-attribute_id_SizeFamily',
        'attributeName': 'Size',
        'valueIds': ['vvc-cls_l3-blinds-shades-attribute_id_SizeFamily-5_7', 'vvc-cls_l3-blinds-shades-attribute_id_SizeFamily-8_9'],
        'valueNames': ['vc-cls_l3-blinds-shades-attribute_id_SizeFamily', 'vc-cls_l3-blinds-shades-attribute_id_SizeFamily'],
      }]
    },
    'fbt': {
      'item': [
        {'slotLocation': 1, 'productId': '1000755816', 'sellPrice': 180},
        {'slotLocation': 2, 'productId': '1000755818', 'sellPrice': 165},
        {'slotLocation': 3, 'productId': '1000755819', 'sellPrice': 165},
      ]
    }
  };

const mockPageLoadPayload: AdobeAnalyticsModels.PageAndModalViewPayload = {
  event: mockEventBlock,
  pageDetail: mockPageDetailDDO,
  productOverview: mockPageOverviewDDO,
  cart: mockCartBlock,
};

describe('Adobe Analytics Service', () => {
  let service: AdobeAnalyticsService;
  let storeMock: Store<globalReducer.State>;

  beforeEach(() => {
    storeMock = jasmine.createSpyObj<Store<globalReducer.State>>('Store', ['select', 'dispatch']);
    service = new AdobeAnalyticsService(storeMock);
  });

  it('should have created', () => {
    expect(service).toBeDefined();
  });

  it('should have those methods', () => {
    expect(service.transmitPageLoadDataToAdobe).toBeDefined();
  });

  it('should be called', () => {
    spyOn(service, 'transmitPageLoadDataToAdobe');
    const result = service.transmitPageLoadDataToAdobe(mockPageLoadPayload);
    expect(service.transmitPageLoadDataToAdobe).toHaveBeenCalled();
  });

  it('should call _trackData when transmitPageLoadDataToAdobe is invoked and DTMReady is true.', () => {
    window['DTMReady'] = true;
    window['_trackData'] = () => {};
    const spy = spyOn<any>(window, '_trackData').and.callThrough();
    service.transmitPageLoadDataToAdobe(mockPageLoadPayload);
    expect(spy).toHaveBeenCalledWith(mockPageLoadPayload);
  });

  it('should not call _trackData when transmitPageLoadDataToAdobe is invoked and DTMReady is false.', () => {
    window['DTMReady'] = false;
    window['_trackData'] = () => {};
    const spy = spyOn<any>(window, '_trackData').and.callThrough();
    service.transmitPageLoadDataToAdobe(mockPageLoadPayload);
    expect(spy).not.toHaveBeenCalled();
  });

  it('should register a NotifyDTMReady callback on window when DTMReady is false.', () => {
    window['DTMReady'] = false;
    window['EventTransceiver'] = {};
    service.transmitPageLoadDataToAdobe(mockPageLoadPayload);
    expect(window['EventTransceiver']['NotifyDTMReady']).toBeDefined();
  });

});
