import { Injectable } from '@angular/core';
import * as models from '../models/adobe-analytics-data-layer-models.interface';
import { Store } from '@ngrx/store';
import { State } from '../../../reducers';
import { AdobeAnalyticsDTMReadyEvent } from '../actions';

/**
 *  Transmits data to Adobe DTM
 */
@Injectable()
export class AdobeAnalyticsService {

  constructor(
    private store: Store<State>
  ) { }

  public transmitDataToAdobe(payload) {
    try {
      window['_trackData'] && window['_trackData'](payload);
    } catch (error) {
      console.log(`ADOBE DTM error: ${error}`);
    }
  }

  private createDTMReadyCallback(payload: models.PageAndModalViewPayload) {
    if (window['EventTransceiver']) {
      window['EventTransceiver']['NotifyDTMReady'] = () => {
        this.store.dispatch(new AdobeAnalyticsDTMReadyEvent(true));
        this.transmitDataToAdobe(payload);
      };
    }
  }

  public transmitPageLoadDataToAdobe(payload: models.PageAndModalViewPayload) {
    // DTM Ready
    !!window['AdobeTracking'] && this.populateAdobeTrackingAfterPageLoad(payload);
    if (!!window['DTMReady']) {
      this.store.dispatch(new AdobeAnalyticsDTMReadyEvent(true));
      this.transmitDataToAdobe(payload);
    } else {
      this.createDTMReadyCallback(payload);
    }
  }

  private populateAdobeTrackingAfterPageLoad(payload: models.PageAndModalViewPayload) {
    if (!!payload) {
      window['AdobeTracking'] = {
        ...window['AdobeTracking'],
        ET: payload
      };
    }
  }

}
