import { Action } from '@ngrx/store';
import { GlobalCartFbtDataInterface } from '../../models/global-cart-add-item-data.interface';
import { GlobalCartErrorData } from '../../models/global-cart-error-data.interface';

export enum GlobalCartAddFbtDataActionTypes {
  AddItemsToCart = '[Add FBT Items To Cart]',
  SaveAddedItemsData = '[Received FBT Add To Cart Data]',
  ShowFbtAddToCartOutcome = '[FBT Add to Cart results received]',
  ResetAddedItemsData = '[FBT Add to Cart results cleared]',
  AddItemsToCartError = '[FBT Add to Cart error]'
}

export class AddItemsToCart implements Action {
  readonly type = GlobalCartAddFbtDataActionTypes.AddItemsToCart;
  constructor(public user: string, public cartId: string, public lang: string, public localizedPOS: string, public products: string) {
  }
}

export class SaveAddedItemsData implements Action {
  readonly type = GlobalCartAddFbtDataActionTypes.SaveAddedItemsData;
  constructor(public payload: GlobalCartFbtDataInterface) {
  }
}

export class ResetAddedItemsData implements Action {
  readonly type = GlobalCartAddFbtDataActionTypes.ResetAddedItemsData;
  constructor() {
  }
}

export class AddItemsToCartError implements Action {
  readonly type = GlobalCartAddFbtDataActionTypes.AddItemsToCartError;
  constructor(public error: GlobalCartErrorData, public hasError: boolean) {
  }
}

export type GlobalCartAddFbtDataActions = AddItemsToCart | SaveAddedItemsData | ResetAddedItemsData | AddItemsToCartError;
