import { Action } from '@ngrx/store';
import { GlobalCartSessionDataInterface } from '../../models/global-cart-session-data.interface';

export enum GlobalCartSessionDataActionTypes {
  CreateAnonymousCartSession = '[Create Anonymous Cart Session]',
  SaveAnonymousCartSession = '[Save Anonymous Cart Session]',
  CreateAccountCartSession = '[Create Account Cart Session]',
  SaveAccountCartSession = '[Save Account Cart Session]',
  SaveCartSessionByCookie = '[Save Cart Session Cookie]'
}

export class CreateAnonymousCartSession implements Action {
  readonly type = GlobalCartSessionDataActionTypes.CreateAnonymousCartSession;
  constructor(public lang: string) {
  }
}

export class SaveAnonymousCartSession implements Action {
  readonly type = GlobalCartSessionDataActionTypes.SaveAnonymousCartSession;
  constructor(public payload: GlobalCartSessionDataInterface) {
  }
}

export class CreateAccountCartSession implements Action {
  readonly type = GlobalCartSessionDataActionTypes.CreateAccountCartSession;
  constructor(public lang: string, public email: string) {
  }
}

export class SaveAccountCartSession implements Action {
  readonly type = GlobalCartSessionDataActionTypes.SaveAccountCartSession;
  constructor(public payload: GlobalCartSessionDataInterface) {
  }
}

export class SaveCartSessionByCookie implements Action {
  readonly type = GlobalCartSessionDataActionTypes.SaveCartSessionByCookie;
  constructor(public cookie: string) {
  }
}

export type GlobalCartSessionDataActions = CreateAnonymousCartSession | SaveAnonymousCartSession | CreateAccountCartSession | SaveAccountCartSession | SaveCartSessionByCookie;
