import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { catchError, switchMap, mergeMap } from 'rxjs/operators';
import { apiUrls } from '../../constants/api.constants';
import { of } from 'rxjs';
import {
  GlobalCartDataActionTypes, ReceivedCartData, GetCurrentCartData, GetCartDataError
} from '../../actions/global-cart-data/global-cart-data.actions';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { OriginService } from '../../../../services/origin/origin.service';
import { ApiParamInfoService } from '@hdca/fe-angular-shared';
import { UpdateCartCount } from '../../../global-header/actions/global-header/global-header-overall/global-header-overall.action';

@Injectable()
export class GlobalCartDataEffects {

  @Effect()
  callAddToCartData$ = this.actions$
    .pipe(
      ofType<GetCurrentCartData>(GlobalCartDataActionTypes.GetCurrentCartData),
      switchMap((action) => {
        const params = this.cookieParams.getApiParamInfo();
        return this.dataFetcher.fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_CURRENT_CART(params.customerId, params.cartId, params.localizedPOS), action.lang).pipe(
          mergeMap((cartData) => {
            return [
              new UpdateCartCount(cartData.totalItemsQuantity),
              new ReceivedCartData(cartData)
            ];
          }),
          catchError(err => {
            // TODO:: use constant
            if (err && err.error.errors[0].type) {
                const payload = err.error.errors[0];
                return of (new GetCartDataError(payload));
            }
            return of ([]);
          })
        );
      })
    );

  constructor(private actions$: Actions, private dataFetcher: DataFetcher, private originService: OriginService, private cookieParams: ApiParamInfoService) {
  }

}
