import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { catchError, map, switchMap } from 'rxjs/operators';
import { apiUrls } from '../../constants/api.constants';
import { of } from 'rxjs';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { OriginService } from '../../../../services/origin/origin.service';
import {
  ErrorPartsAndServices,
  GetPartsAndServices,
  GlobalCartPartsAndServicesActionTypes, SuccessPartsAndServices
} from '../../actions/global-cart-parts-and-services-data/global-cart-parts-and-services-data.actions';
import { AddApplianceItemToCart } from '../../actions/global-cart-add-appliance-item-data/global-cart-add-appliance-item-data.actions';
import { GlobalCartService } from '../../services/global-cart.service';
import { LanguageService } from '@hdca/fe-angular-shared';
import { GlobalCartPartsAndServicesDataInterface } from '../../models/global-cart-parts-and-services-data.interface';

@Injectable()
export class GlobalCartPartsAndServicesDataEffects {

  @Effect()
  getPartsAndServices$ = this.actions$
    .pipe(
      ofType<GetPartsAndServices>(GlobalCartPartsAndServicesActionTypes.GetPartsAndServices),
      switchMap((action) => {
        return this.dataFetcher.fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_PARTS_AND_SERVICES(action.productCode, action.postalCode, action.localizedPOS), action.lang).pipe(
          map((partsData: GlobalCartPartsAndServicesDataInterface) => {
            if (!!action.installServiceCode) {
              const installServiceAddon = {
                selected: true,
                required: true,
                product: {
                  code: action.installServiceCode
                },
                source: ''
              };
              partsData.applianceAddons.push(installServiceAddon);
            }
            return new SuccessPartsAndServices(partsData, action.postalCode, action.localizedPOS, !!action.installServiceCode ? false : action.addOnAvailable);
          }),
          catchError(error => {
            return of(new ErrorPartsAndServices());
          })
        );
      })
    );

  @Effect()
  savePartsAndServices$ = this.actions$
    .pipe(
      ofType<SuccessPartsAndServices>(GlobalCartPartsAndServicesActionTypes.SuccessPartsAndServices),
      map(action => new AddApplianceItemToCart(this.globalCartService.getUser(), this.globalCartService.getCartId(), action.postalCode, this.langService.getLanguage(), action.localizedPOS, action.payload, action.addOnAvailable))
    );

  constructor(private actions$: Actions, private dataFetcher: DataFetcher, private originService: OriginService, private globalCartService: GlobalCartService, private langService: LanguageService) {
  }

}
