import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { catchError, map, switchMap, tap, filter } from 'rxjs/operators';
import { apiUrls } from '../../constants/api.constants';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { OriginService } from '../../../../services/origin/origin.service';
import {
  ErrorPaypalToken, GetConfiguration,
  GetPaypalToken,
  GlobalCartPaypalDataActionTypes, SavePaypalToken
} from '../../actions/global-cart-paypal-data/global-cart-paypal-data.actions';
import { GlobalCartService } from '../../services/global-cart.service';
import { of } from 'rxjs';

@Injectable()
export class GlobalCartPaypalDataEffects {

  @Effect()
  getConfiguration$ = this.actions$
    .pipe(
      ofType<GetConfiguration>(GlobalCartPaypalDataActionTypes.GetConfiguration),
      switchMap((action) => {
        return this.dataFetcher.fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_CONFIGURATIONS(), action.language).pipe(
          map(payload => new GetPaypalToken(payload, action.language)),
          catchError(error => {
            return of(new ErrorPaypalToken());
          })
        );
      })
    );

  @Effect()
  getPaypalToken$ = this.actions$
    .pipe(
      ofType<GetPaypalToken>(GlobalCartPaypalDataActionTypes.GetPaypalToken),
      switchMap((action) => {
        return this.dataFetcher.fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_PAYPAL_TOKEN(this.globalCartService.getUser(), this.globalCartService.getCartId()), action.language).pipe(
          map(payload => new SavePaypalToken(payload, action.config, action.language)),
          catchError(error => {
            return of(new ErrorPaypalToken());
          })
        );
      })
    );

  @Effect({dispatch: false})
  savePaypalToken$ = this.actions$
    .pipe(
      ofType<SavePaypalToken>(GlobalCartPaypalDataActionTypes.SavePaypalToken),
      tap(action => window.location.href = action.config.paypalUrl + action.config.paypalQueryParams + action.payload.token)
    );

  constructor(private actions$: Actions, private dataFetcher: DataFetcher, private originService: OriginService, private globalCartService: GlobalCartService) {
  }

}
