import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import {
  CreateAccountCartSession,
  CreateAnonymousCartSession,
  GlobalCartSessionDataActionTypes, SaveAccountCartSession, SaveAnonymousCartSession, SaveCartSessionByCookie
} from '../../actions/global-cart-session-data/global-cart-session-data.actions';
import { catchError, map, switchMap, tap } from 'rxjs/operators';
import { HttpHeaders } from '@angular/common/http';
import { apiUrls } from '../../constants/api.constants';
import { of } from 'rxjs';
import { OriginService } from '../../../../services/origin/origin.service';
import { DataSender } from '../../../../services/data-sender/data-sender.service';
import { CookiesService } from '@hdca/fe-angular-shared';
import { COOKIE_CONSTANTS } from '../../constants/cookie.constants';

@Injectable()
export class GlobalCartSessionDataEffects {

  @Effect()
  createAnonymousCartSession$ = this.actions$
    .pipe(
      ofType<CreateAnonymousCartSession>(GlobalCartSessionDataActionTypes.CreateAnonymousCartSession),
      switchMap((action) => {
        const headers = new HttpHeaders({
          'Content-Type': 'application/x-www-form-urlencoded'
        });

        return this.dataSender.post(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.CREATE_CART_SESSION(), action.lang, '', headers).pipe(
          map(cartSessionData => {
            return new SaveAnonymousCartSession(cartSessionData);
          }),
          catchError(error => {
            return of(error);
          })
        );
      })
    );

  @Effect()
  createAccountCartSession$ = this.actions$
  .pipe(
    ofType<CreateAccountCartSession>(GlobalCartSessionDataActionTypes.CreateAccountCartSession),
    switchMap((action) => {
      const headers = new HttpHeaders({
        'Content-Type': 'application/x-www-form-urlencoded'
      });

      return this.dataSender.post(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.CREATE_ACCOUNT_CART_SESSION(action.email), action.lang, '', headers).pipe(
        map(cartSessionData => {
          return new SaveAccountCartSession(cartSessionData);
        }),
        catchError(error => {
          return of(error);
        })
      );
    })
  );

  @Effect({dispatch: false})
  saveAnonymousCartSession$ = this.actions$
    .pipe(
      ofType<SaveAnonymousCartSession>(GlobalCartSessionDataActionTypes.SaveAnonymousCartSession),
      tap(action => {
        if (!!action.payload.guid) {
          this.cookiesService.setCookie(COOKIE_CONSTANTS.CART, action.payload.guid);
        }
      })
    );

  @Effect({dispatch: false})
  saveAccountCartSession$ = this.actions$
  .pipe(
    ofType<SaveAccountCartSession>(GlobalCartSessionDataActionTypes.SaveAccountCartSession),
    tap(action => {
      if (!!action.payload.guid) {
        this.cookiesService.setCookie(COOKIE_CONSTANTS.CART, action.payload.code);
      }
    })
  );

  constructor(private actions$: Actions, private dataSender: DataSender, private originService: OriginService, private cookiesService: CookiesService) {
  }

}
