import { Injectable } from '@angular/core';
import { Effect, Actions } from '@ngrx/effects';
import { Action } from '@ngrx/store';
import { Observable, of } from 'rxjs';
import * as recommendationActions from '../../actions/recommendations/recommendation.actions';
import { apiUrls } from '../../constants/api.constants';
import { switchMap, tap } from 'rxjs/operators';
import { IRecommendationItem } from '../../models/recommendations.interface';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { OriginService } from '../../../../services/origin/origin.service';

@Injectable()
export class RecommendationsEffects {

    @Effect()
    getRecommendations$: Observable<Action> = this.actions$
        .ofType<recommendationActions.GetRecommendations>(recommendationActions.RecommendationActionTypes.GetRecommendations)
        .pipe(
        switchMap((action) => {
            return this.serviceData
                .fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_RECOMMENDATIONS(action.payload.productId, action.payload.scheme, action.payload.relationship, action.payload.trackingId, action.payload.localizedPOS, action.payload.fields, action.payload.catalogVersion), action.payload.lang)
                .map((response: any) => {
                    const recommendations: IRecommendationItem[] =   response && response.schemes && response.schemes[0]
                    && response.schemes[0].items ? response.schemes[0].items : [];
                    return new recommendationActions.GetRecommendationsSuccess(recommendations);
                })
                .catch(err => {
                    console.log(err);
                    return of();
                }
            );
        }));

        constructor(
            private actions$: Actions,
            private serviceData: DataFetcher,
            private originService: OriginService
        ) { }

}
