import { Inject, NgModule, PLATFORM_ID } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HttpClient } from '@angular/common/http';
import { TransferState } from '@angular/platform-browser';
import { TranslatesBrowserLoaderService } from '../../modules/translation/translation-browser/translation-browser-loader.service';
import { TranslateLoader, TranslateModule } from '@ngx-translate/core';
import { LanguageService } from '@hdca/fe-angular-shared';
import { GlobalAddToCartSuccessContainer } from './containers/global-add-to-cart-success/global-add-to-cart-success.container';
import { StoreModule } from '@ngrx/store';
import { globalCartReducers } from './reducers/state';
import { EffectsModule } from '@ngrx/effects';
import { GlobalCartDataEffects } from './effects/global-cart-data/global-cart-data.effects';
import { GlobalCartContainer } from './containers/global-cart/global-cart.container';
import { GlobalCartService } from './services/global-cart.service';
import { GlobalCartSessionDataEffects } from './effects/global-cart-session/global-cart-session-data.effects';
import { GlobalCartAddItemDataEffects } from './effects/global-cart-add-item-data/global-cart-add-item-data.effects';
import { GlobalCartAddFbtDataEffects } from './effects/global-cart-add-item-data/global-cart-add-fbt-data.effects';
import { GlobalAddToCartSuccessProductComponent } from './components/global-add-to-cart-success-product/global-add-to-cart-success-product.component';
import { GlobalAddToCartSuccessMinicartComponent } from './components/global-add-to-cart-success-minicart/global-add-to-cart-success-minicart.component';
import { GlobalCartPaypalDataEffects } from './effects/global-cart-paypal-data/global-cart-paypal-data.effects';
// TODO: Remove cross-referencing
import { GlobalCartAddApplianceItemDataEffects } from './effects/global-cart-add-appliance-item-data/global-cart-add-appliance-item-data.effects';
import { GlobalCartPartsAndServicesDataEffects } from './effects/global-cart-parts-and-services-data/global-cart-parts-and-services-data.effects';
import { RecommendationsEffects } from './effects/recommendations/recommendations.effects';
import { GlobalAddToCartCertonaComponent } from './components/global-add-to-cart-recommendations/global-add-to-cart-recommendations.component';
import { AngularClModule } from '@hdca/fe-angular-cl';

export function translateStaticLoaderGlobalCart(http: HttpClient, transferState: TransferState): TranslatesBrowserLoaderService {
  return new TranslatesBrowserLoaderService('/node-assets/i18n/global-cart/', '.json', transferState, http);
}

@NgModule({
  imports: [
    CommonModule,
    StoreModule.forFeature('globalCart', globalCartReducers),
    AngularClModule.forRoot(),
    EffectsModule.forFeature([
      GlobalCartDataEffects,
      GlobalCartSessionDataEffects,
      GlobalCartAddItemDataEffects,
      GlobalCartAddFbtDataEffects,
      GlobalCartPaypalDataEffects,
      GlobalCartPartsAndServicesDataEffects,
      GlobalCartAddApplianceItemDataEffects,
      RecommendationsEffects
    ]),
    TranslateModule.forChild({
      loader: {
        provide: TranslateLoader,
        useFactory: translateStaticLoaderGlobalCart,
        deps: [HttpClient, TransferState]
      }
    }),
  ],
  providers: [
    GlobalCartService
  ],
  declarations: [
    GlobalCartContainer,
    GlobalAddToCartSuccessContainer,
    GlobalAddToCartSuccessProductComponent,
    GlobalAddToCartSuccessMinicartComponent,
    GlobalAddToCartCertonaComponent
  ],
  exports: [
    GlobalCartContainer
  ]
})
/**
 * @description - This core module contains the code necessary for cart related functions. The GlobalCartService contains methods to retrieve current cart data.
 */
export class HDCAGlobalCartModule {
  constructor(private languageService: LanguageService, @Inject(PLATFORM_ID) platformId: any) {
    const lang = this.languageService.getLanguage();
    this.languageService.setLanguage(lang);
  }
}
