export interface GlobalCartAddItemDataInterface {
  entry: Entry;
  quantity: number;
  quantityAdded: number;
  statusCode: string;
  message?: string;
}

export interface GlobalCartFbtDataInterface {
  cartModifications: GlobalCartAddItemDataInterface[];
}

export interface BasePrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface BasePriceValue {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface Country {
  isocode: string;
  name: string;
}

export interface Region {
  countryIso: string;
  isocode: string;
  isocodeShort: string;
  name: string;
}

export interface Address {
  billingAddress: boolean;
  country: Country;
  defaultAddress: boolean;
  eveningPhoneType: string;
  formattedAddress: string;
  id: string;
  line1: string;
  paypalAddress: boolean;
  phone: string;
  postalCode: string;
  primaryPhoneType: string;
  region: Region;
  secondaryPhoneType: string;
  shippingAddress: boolean;
  town: string;
  visibleInAddressBook: boolean;
}

export interface GeoPoint {
  latitude: number;
  longitude: number;
}

export interface ClosingTime {
  formattedHour: string;
  hour: number;
  minute: number;
}

export interface OpeningTime {
  formattedHour: string;
  hour: number;
  minute: number;
}

export interface WeekDayOpeningList {
  closingTime: ClosingTime;
  openingTime: OpeningTime;
  closed: boolean;
  weekDay: string;
}

export interface OpeningHours {
  code: string;
  weekDayOpeningList: WeekDayOpeningList[];
}

export interface DeliveryPointOfService {
  address: Address;
  description: string;
  displayName: string;
  geoPoint: GeoPoint;
  name: string;
  openingHours: OpeningHours;
}

export interface DeliveryStoreStock {
  stockLevel: number;
  stockLevelStatus: string;
}

export interface DisplayPrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface StoreStock {
  stockLevel: number;
  stockLevelStatus: string;
}

export interface PickupInStore {
  changeStore: boolean;
  selected: boolean;
  storeStock: StoreStock;
}

export interface FulfillmentOptions {
  pickupInStore: PickupInStore;
}

export interface OnlineStock {
  stockLevel: number;
  stockLevelStatus: string;
}

export interface Category {
  code: string;
  name: string;
}

export interface Image {
  altText: string;
  format: string;
  imageType: string;
  url: string;
}

export interface DisplayPrice2 {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface Regprice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface OptimizedPrice {
  availabilityMsg: string;
  displayPrice: DisplayPrice2;
  endDate: Date;
  productId: string;
  productStatus: string;
  regprice: Regprice;
  storeId: string;
}

export interface Price {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface Stock {
  stockLevel: number;
  stockLevelStatus: string;
}

export interface Product {
  articleType: string;
  availableForPickup: boolean;
  bopis: boolean;
  buyable: boolean;
  categories: Category[];
  code: string;
  deliveryTime: string;
  deliveryUnit: string;
  depotDirect: boolean;
  description: string;
  ecoFeeEligible: boolean;
  freeShipping: number;
  images: Image[];
  manufacturer: string;
  modelNumber: string;
  name: string;
  optimizedPrice: OptimizedPrice;
  price: Price;
  purchasable: boolean;
  sellableIntent: string;
  shipToHome: boolean;
  shipToStoreTime: string;
  shipToStoreUnit: string;
  stock: Stock;
  unitOfMeasure: string;
  url: string;
}

export interface ShippingFee {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

export interface TotalPrice {
  currencyIso: string;
  formattedValue: string;
  priceType: string;
  value: number;
}

interface Documents {
  URL: string;
  mime: string;
  name: string;
}

export interface Entry {
  applianceVendorNumber: string;
  basePrice: BasePrice;
  basePriceValue: BasePriceValue;
  deliveryPointOfService: DeliveryPointOfService;
  deliveryStoreStock: DeliveryStoreStock;
  displayPrice: DisplayPrice;
  entryNumber: number;
  fulfillmentOptions: FulfillmentOptions;
  isApplianceOrAddon: boolean;
  isFreeGift: boolean;
  onlineStock: OnlineStock;
  proDirectQualifier: string;
  product: Product;
  productStatus: string;
  quantity: number;
  requiresProductAddons: boolean;
  shippingFee: ShippingFee;
  totalPrice: TotalPrice;
  totalStockQuantity: number;
  updateable: boolean;
}

// interface ModifiedProductOverall {
//   entry: ModifiedProductEntry;
//   message?: string;
//   quantity: number;
//   quantityAdded: number;
//   statusCode: string;
// }

// interface Categories {
//   code: string;
//   name: string;
// }

// interface ModifiedProductEntry {
//   displayPrice?: DisplayPrice;
//   entryNumber: number;
//   isApplianceOrAddon?: boolean;
//   isFreeGift?: boolean;
//   onlineStock?: OnlineStock;
//   proDirectQualifier?: string;
//   product: ModifiedProduct;
//   quantity?: number;
//   requiresProductAddons?: boolean;
//   shippingFee?: ShippingFee;
//   totalPrice?: TotalPrice;
// }

// interface ModifiedProduct {
//   articleType?: string;
//   availableForPickup?: boolean;
//   bopis?: boolean;
//   buyable?: boolean;
//   code: string;
//   deliveryTime?: string;
//   deliveryUnit?: string;
//   depotDirect?: boolean;
//   description?: string;
//   ecoFeeEligible?: boolean;
//   freeShipping?: number;
//   images: AlternateImage[];
//   manufacturer: string;
//   modelNumber: string;
//   name: string;
//   optimizedPrice?: OptimizedPrice;
//   price?: Price;
//   purchasable?: boolean;
//   sellableIntent?: string;
//   shipToHome?: boolean;
//   shipToStoreTime?: string;
//   shipToStoreUnit?: string;
//   stock?: Stock;
//   unitOfMeasure?: string;
//   url: string;
//   categories?: Categories[];
//   documents?: Documents[];
// }

export interface SuccessModalMessage {
  imageUrl: string;
  imageAltText: string;
  text: string;
}
