import { GlobalCartFbtDataInterface } from '../../models/global-cart-add-item-data.interface';
import {
  GlobalCartAddFbtDataActions,
  GlobalCartAddFbtDataActionTypes
} from '../../actions/global-cart-add-item-data/global-cart-add-fbt-data.actions';
import { GlobalCartErrorData } from '../../models/global-cart-error-data.interface';

export interface GlobalCartAddFbtDataState extends GlobalCartFbtDataInterface {
  loading: boolean;
  error: GlobalCartErrorData;
}

export const initialState: GlobalCartAddFbtDataState = {
  loading: undefined,
  cartModifications: undefined,
  error: {
    errorObj: undefined,
    hasError: undefined
  }
};

export function globalCartAddFbtDataReducer(state = initialState, action: GlobalCartAddFbtDataActions): GlobalCartAddFbtDataState {

  switch (action.type) {
    case GlobalCartAddFbtDataActionTypes.AddItemsToCart:
      return {
        ...state,
        loading: true,
        error: {
          errorObj: undefined,
          hasError: undefined
        }
      };

    case GlobalCartAddFbtDataActionTypes.SaveAddedItemsData:
      return {
        ...state,
        ...action.payload,
        loading: false,
        error: {
          errorObj: undefined,
          hasError: false
        }
      };

    case GlobalCartAddFbtDataActionTypes.ResetAddedItemsData:
      return { ...state, cartModifications: undefined, loading: false };

    case GlobalCartAddFbtDataActionTypes.AddItemsToCartError:
      return {
        ...state,
        loading: false,
        error: { ...state.error, hasError: action.hasError, errorObj: action.error },
      };

    default: {
      return state;
    }
  }
}
