import { GlobalCartAddItemDataInterface } from '../../models/global-cart-add-item-data.interface';
import {
  GlobalCartAddItemDataActions,
  GlobalCartAddItemDataActionTypes
} from '../../actions/global-cart-add-item-data/global-cart-add-item-data.actions';
import { GlobalCartErrorData } from '../../models/global-cart-error-data.interface';

export interface GlobalCartAddItemDataState extends GlobalCartAddItemDataInterface {
  loading: boolean;
  error: GlobalCartErrorData;
}

export const addItemInitialState: GlobalCartAddItemDataState = {
  loading: undefined,
  entry: undefined,
  quantity: undefined,
  quantityAdded: undefined,
  statusCode: undefined,
  error: {
    errorObj: undefined,
    hasError: undefined
  }
};

export function globalCartAddItemDataReducer(state = addItemInitialState, action: GlobalCartAddItemDataActions): GlobalCartAddItemDataState {

  switch (action.type) {
    case GlobalCartAddItemDataActionTypes.AddItemToCart:
      return {
        ...state,
        loading: true,
        error: {
          errorObj: undefined,
          hasError: undefined
        }
      };

    case GlobalCartAddItemDataActionTypes.SaveAddedItemData:
      return {
        ...state,
        ...action.payload,
        loading: false,
        error: {
          errorObj: undefined,
          hasError: false
        }
      };

    case GlobalCartAddItemDataActionTypes.AddItemError:
      return {
        ...state,
        loading: false,
        error: {
          ...state.error,
          hasError: action.hasError,
          errorObj: action.error
        },
      };

    default: {
      return state;
    }
  }
}
