import { GlobalCartDataInterface } from '../../models/global-cart-data.interface';
import {
  GlobalCartDataActions,
  GlobalCartDataActionTypes
} from '../../actions/global-cart-data/global-cart-data.actions';
import { GlobalCartErrorData } from '../../models/global-cart-error-data.interface';

export interface GlobalCartDataState extends GlobalCartDataInterface {
  loading: boolean;
  error: GlobalCartErrorData;
}

export const initialState: GlobalCartDataState = {
  loading: undefined,
  type: undefined,
  applianceDeliveryCost: undefined,
  applianceItemsQuantity: undefined,
  bodfsItemsQuantity: undefined,
  bopisItemsQuantity: undefined,
  bossItemsQuantity: undefined,
  code: undefined,
  deliveryItemsQuantity: undefined,
  deliveryOrderGroups: undefined,
  entries: undefined,
  guid: undefined,
  isCartBuyable: undefined,
  isRuralPostalCode: undefined,
  isShippingFree: undefined,
  numberOfProducts: undefined,
  subTotal: undefined,
  totalItemsQuantity: undefined,
  totalPriceWithTax: undefined,
  totalSavings: undefined,
  totalTax: undefined,
  isCheckoutButtonEnabled: undefined,
  appliancePostalCode: undefined,
  error: {
    errorObj: undefined
  }
};

export function globalCartDataReducer(state = initialState, action: GlobalCartDataActions): GlobalCartDataState {

  switch (action.type) {
    case GlobalCartDataActionTypes.GetCurrentCartData:
      return { ...state, loading: true };

    case GlobalCartDataActionTypes.ReceivedCartData:
      return { ...state, ...action.payload, loading: false, error: initialState.error };

    case GlobalCartDataActionTypes.GetCartDataError:
      return { ...state, loading: false, error: { errorObj: action.error } };

    default: {
      return state;
    }
  }
}
