// ngrx imports
import { Action } from '@ngrx/store';
// Specify constants for global header my account
export const CHANGED_EMAIL_ADDRESS = '[my account] changed email address';
export const CHANGED_PASSWORD = '[my account] changed password';
export const CHANGED_PASSWORD_VISIBILITY = '[my account] changed password visibility';
export const RESET_LOGIN_ERRORS = '[my account] reset login errors';
export const VALIDATED_LOGIN_CREDENTIALS = '[my account] validated login credentials';
export const INITIATE_LOGIN_PROCESS = '[my account] initiate login process';
export const INITIATE_LOGOUT_PROCESS = '[my account] initiate logout process';
export const LOGIN_RESPONSE = '[my account] login success';
export const LOGIN_ERROR = '[my account] login error';
export const REMEMBER_ME_CHECK = '[my account] remember me check';
export const REMEMBER_ME_TOGGLE = '[my account] remember me toggle';
export const LOGOUT_ERROR = '[my account] log out success';
export const LOGOUT_SUCCESS = '[my account] log out error';
export const EXPIRED_USER = '[my account] expired user session';

// Model Imports
import { LoginFormData, LoginHeaderData } from '../../../models/global-header-account.interface';

// Actions
export class ChangedEmailAddress implements Action {
  readonly type = CHANGED_EMAIL_ADDRESS;
  constructor(public emailAddress: string, public emailError: boolean) { }
}

export class RememberMeCheck implements Action {
  readonly type = REMEMBER_ME_CHECK;
  constructor(public savedEmailAddress: string) { }
}

export class RememberMeToggle implements Action {
  readonly type = REMEMBER_ME_TOGGLE;
  constructor(public checked: boolean) { }
}

export class ChangedPassword implements Action {
  readonly type = CHANGED_PASSWORD;
  constructor(public password: string, public passwordError: boolean) { }
}

export class ChangePasswordVisibility implements Action {
  readonly type = CHANGED_PASSWORD_VISIBILITY;
  constructor(public passwordVisible: boolean) { }
}

export class ResetLoginErrors implements Action {
  readonly type = RESET_LOGIN_ERRORS;
  constructor(public emailError: boolean, public passwordError: boolean) { }
}

export class ValidatedLoginCredentials implements Action {
  readonly type = VALIDATED_LOGIN_CREDENTIALS;
  constructor(public emailError: boolean, public passwordError: boolean, public hasValidationCompleted: boolean) { }
}

export class InitiateLoginProcess implements Action {
  readonly type = INITIATE_LOGIN_PROCESS;
  constructor(public payload: LoginFormData, public headers: LoginHeaderData, public lang: string) { }
}

export class InitiateLogoutProcess implements Action {
  readonly type = INITIATE_LOGOUT_PROCESS;
  constructor(public headers: LoginHeaderData, public lang: string) { }
}

export class LoginResponse implements Action {
  readonly type = LOGIN_RESPONSE;
  constructor(public code: string, public success: boolean) { }
}

export class LoginError implements Action {
  readonly type = LOGIN_ERROR;
  constructor(public error: any) { }
}

export class LogoutSuccess implements Action {
  readonly type = LOGOUT_SUCCESS;
  constructor() { }
}

export class LogoutError implements Action {
  readonly type = LOGOUT_ERROR;
  constructor() { }
}

export class SetExpiredUser implements Action {
  readonly type = EXPIRED_USER;
  constructor() { }
}

export type GlobalHeaderAccountActions = ChangedEmailAddress
  | ChangedPassword
  | ResetLoginErrors
  | ValidatedLoginCredentials
  | InitiateLoginProcess
  | InitiateLogoutProcess
  | LoginResponse
  | LoginError
  | ChangePasswordVisibility
  | RememberMeCheck
  | RememberMeToggle
  | LogoutError
  | LogoutSuccess
  | SetExpiredUser;
