// models
import { GlobalHeaderLocalizeUI, GlobalHeaderLocalizeAkamai, GlobalHeaderLocalizeHours, GlobalHeaderLocalizeStoreSelector, GlobalHeaderLocalizationUI, LocalizationData} from '../../../models/global-header-localize.interface';
import { GlobalHeaderLocalize } from '../../../reducers/global-header-localize/global-header-localize.reducer';

import { Action } from '@ngrx/store';

export const LOAD_LOCALIZE = '...LOADING Global Header Localize';
export const LOAD_AUTO_LOCALIZE = '...LOADING Global Header Auto Localize';
export const LOAD_AUTO_LOCALIZATION = '...LOADING Global Header Auto Localization';
export const LOAD_COMMERCE_STORE = '...LOADING Global Header Commerce Store';
export const LOAD_COMMERCE_STORE_LIST = '...LOADING Global Header Commerce Store List';
export const UNLOCALIZE = 'Unlocalize';
export const UPDATE_GLOBAL_HEADER_LOCALIZE = '...UPDATE Global Header Localize';
export const UPDATE_GLOBAL_HEADER_LOCALIZE_AKAMAI = '...UPDATE Global Header Localize Akamai';
export const UPDATE_GLOBAL_HEADER_LOCALIZE_HOURS = '...UPDATE Global Header Localize Hours';
export const UPDATE_GLOBAL_HEADER_LOCALIZE_STORE_SELECTOR = '...UPDATE Global Header Localize Store Selector';
export const TOGGLE_GLOBAL_HEADER_LOCALIZE_UI = '...UPDATE Global Header Localize UI';
export const PICK_UP_STORE_ID_RECEIVED = '...UPDATE PickUp Store Id';
export const STORE_ID_RECEIVED = '...UPDATE Store Id';
export const UPDATE_GLOBAL_HEADER_LOCALIZATION_UI = '...UPDATE Global Header Localize Nearest Store UI';

export class LoadLocalize implements Action {
  readonly type = LOAD_LOCALIZE;
  constructor(public lang: string, public params, public userId: string) {}
}

export class LoadAutoLocalize implements Action {
  readonly type = LOAD_AUTO_LOCALIZE;
  constructor(public lang: string, public params, public userId: string, public isAkamaiPopupEnabled: boolean) {}
}

export class LoadAutoLocalization implements Action {
    readonly type = LOAD_AUTO_LOCALIZATION;
    constructor(public lang: string, public params: LocalizationData, public userId: string) {}
}

export class LoadCommerceStore implements Action {
  readonly type = LOAD_COMMERCE_STORE;
  constructor(public lang: string, public query: string) { }
}

export class LoadCommerceStoreList implements Action {
  readonly type = LOAD_COMMERCE_STORE_LIST;
  constructor(public pageSize: string, public currentPage: string, public query: string, public lang: string, public latitude?: number, public longitude?: number, public isInGlobalHeader: boolean = true) { }
}

export class Unlocalize implements Action {
  readonly type = UNLOCALIZE;
  constructor() { }
}

export class UpdateGlobalHeaderLocalize implements Action {
  readonly type = UPDATE_GLOBAL_HEADER_LOCALIZE;
  constructor(public newState: GlobalHeaderLocalize) {}
}

export class UpdateGlobalHeaderLocalizeAkamai implements Action {
  readonly type = UPDATE_GLOBAL_HEADER_LOCALIZE_AKAMAI;
  constructor(public newState: GlobalHeaderLocalizeAkamai) {}
}

export class UpdateGlobalHeaderLocalizeHours implements Action {
  readonly type = UPDATE_GLOBAL_HEADER_LOCALIZE_HOURS;
  constructor(public newState: GlobalHeaderLocalizeHours) {}
}

export class UpdateGlobalHeaderLocalizeStoreSelector implements Action {
  readonly type = UPDATE_GLOBAL_HEADER_LOCALIZE_STORE_SELECTOR;
  constructor(public newState: GlobalHeaderLocalizeStoreSelector) {}
}

export class ToggleGlobalHeaderLocalizeUI implements Action {
  readonly type = TOGGLE_GLOBAL_HEADER_LOCALIZE_UI;
  constructor(public newState: GlobalHeaderLocalizeUI) {}
}

export class PickUpStoreIdReceived implements Action {
  readonly type = PICK_UP_STORE_ID_RECEIVED;
  constructor(public pickUpStoreId: string) {
  }
}

export class StoreIdReceived implements Action {
  readonly type = STORE_ID_RECEIVED;
  constructor(public storeId: string) {
  }
}

export class UpdateGlobalHeaderLocalizationUI implements Action {
    readonly type = UPDATE_GLOBAL_HEADER_LOCALIZATION_UI;
    constructor(public newState: GlobalHeaderLocalizationUI) {}
}

export type GlobalHeaderLocalizeActions = LoadLocalize | LoadAutoLocalize | LoadCommerceStoreList | Unlocalize | LoadCommerceStore | UpdateGlobalHeaderLocalize | UpdateGlobalHeaderLocalizeAkamai | UpdateGlobalHeaderLocalizeHours | UpdateGlobalHeaderLocalizeStoreSelector | ToggleGlobalHeaderLocalizeUI | PickUpStoreIdReceived | StoreIdReceived | UpdateGlobalHeaderLocalizationUI | LoadAutoLocalization;
