import { Action } from '@ngrx/store';
// Import models
import { SearchKeywordSuggestions, SearchAutosuggestDetails } from '../../../models/global-header-search.interface';

// Specify constants for search result and keywords
export const LOAD_SEARCH_RESULTS = '[search] Load search results';
export const LOAD_SEARCH_TOP_RESULTS = '[search] Load search top results';
export const LOAD_SEARCH_KEYWORD_SUGGESTIONS = '[search] Load search keywords';
export const SEARCH_TOP_RESULTS_SUCCESS = '[search] top results success';
export const SEARCH_KEYWORD_KEYWORD_SUGGESTIONS_SUCCESS = '[search] keywords success';
export const SEARCH_TOP_RESULTS_ERROR = '[search] top results error';
export const SEARCH_KEYWORD_KEYWORD_SUGGESTIONS_ERROR = '[search] keywords error';
export const CLEAR_SEARCH_TOP_RESULTS = '[search] top results cleared';
export const CLEAR_SEARCH_KEYWORD_SUGGESTIONS = '[search] keywords cleared';
export const SEARCH_TERM_CHANGED = '[search] search term changed';
export const SEARCH_KEYWORD_SELECTION_CHANGED = '[search] search keyword selection changed';
export const HIDE_AUTO_SUGGEST = '[search] hide autosuggest';
export const HIDE_AUTO_SUGGEST_RETAIN_SEARCH_TERM = '[search] hide autosuggest retain search term';
export const SEARCH_REDIRECT = '[search] redirect';

export class LoadSearchResults implements Action {
  readonly type = LOAD_SEARCH_RESULTS;
  constructor(public lang: string, public query: string, public localizedPOS: string) { }
}

export class LoadTopSearchResults implements Action {
  readonly type = LOAD_SEARCH_TOP_RESULTS;
  constructor(public catalogVersion: string, public lang: string, public query: string, public localizedPOS: string, public update: boolean) { }
}

export class LoadSearchKeywords implements Action {
  readonly type = LOAD_SEARCH_KEYWORD_SUGGESTIONS;
  constructor(public catalogVersion: string, public lang: string, public query: string) { }
}

export class ClearTopSearchResults implements Action {
  readonly type = CLEAR_SEARCH_TOP_RESULTS;
  constructor(public catalogVersion: string, public lang: string, public query: string) { }
}

export class ClearSearchKeywords implements Action {
  readonly type = CLEAR_SEARCH_KEYWORD_SUGGESTIONS;
  constructor(public catalogVersion: string, public lang: string, public query: string) { }
}

export class LoadTopSearchResultsSuccess implements Action {
    readonly type = SEARCH_TOP_RESULTS_SUCCESS;
    constructor(public payload: SearchAutosuggestDetails, public update: boolean) { }
}

export class LoadSearchKeywordsSuccess implements Action {
    readonly type = SEARCH_KEYWORD_KEYWORD_SUGGESTIONS_SUCCESS;
    constructor(public payload: SearchKeywordSuggestions) { }
}

export class LoadTopSearchResultsError implements Action {
  readonly type = SEARCH_TOP_RESULTS_ERROR;
  constructor(public payload: { error: Error }) { }
}

export class LoadSearchKeywordsError implements Action {
  readonly type = SEARCH_KEYWORD_KEYWORD_SUGGESTIONS_ERROR;
  constructor(public payload: { error: Error }) { }
}

export class SearchTermChanged implements Action {
  readonly type = SEARCH_TERM_CHANGED;
  constructor(public payload: string) { }
}

export class SearchKeywordSelectionChanged implements Action {
  readonly type = SEARCH_KEYWORD_SELECTION_CHANGED;
  constructor(public selectedIndex: number, public keyword: string, public hover: boolean) { }
}

export class HideAutoSuggest implements Action {
  readonly type = HIDE_AUTO_SUGGEST;
  constructor(public payload: boolean) { }
}

export class HideAutoSuggestRetainSearchTerm implements Action {
  // Used for hiding the autosuggest when changing text in input field
  // Need to reset Autosuggest container contents but retain text in input field to reduce interaction burden on user
  // - They can easily do a new search without having to type a brand new word from scratch
  readonly type = HIDE_AUTO_SUGGEST_RETAIN_SEARCH_TERM;
  constructor(public payload: boolean) { }
}

export class SearchRedirect implements Action {
  readonly type = SEARCH_REDIRECT;
  constructor(public url: string) { }
}

export type GlobalSearchActions = LoadSearchResults
  | LoadTopSearchResults
  | LoadSearchKeywords
  | LoadTopSearchResultsSuccess
  | LoadSearchKeywordsSuccess
  | ClearTopSearchResults
  | ClearSearchKeywords
  | SearchTermChanged
  | SearchKeywordSelectionChanged
  | HideAutoSuggest
  | HideAutoSuggestRetainSearchTerm
  | LoadTopSearchResultsError
  | LoadSearchKeywordsError
  | SearchRedirect;
