// Core imports
import { Component, OnInit, EventEmitter, Output, Input } from '@angular/core';
// Rxjs imports
import { Observable } from 'rxjs';
// Model imports
import { AuthenticationState } from '../../../models/global-header-account.interface';

@Component({
  selector: '[global-header-account-sign-in]',
  templateUrl: './global-header-account-sign-in.component.html'
})
export class GlobalHeaderAccountSignInComponent implements OnInit {
  // Event emitters for communicating with parent container that form inputs have changed or form is being submitted
  @Output() emailHasChanged: EventEmitter<any> = new EventEmitter();
  @Output() passwordHasChanged: EventEmitter<any> = new EventEmitter();
  @Output() formSubmitHasBegun: EventEmitter<any> = new EventEmitter();
  @Output() passwordVisibilityHasBeenToggled: EventEmitter<any> = new EventEmitter();
  @Output() rememberMeChecked: EventEmitter<any> = new EventEmitter();
  // Inputs from parent container for updating UI of sign in form
  @Input() hasPasswordError: boolean;
  @Input() hasEmailError: boolean;
  @Input() isLoggingIn: boolean;
  @Input() isPasswordShown: boolean;
  @Input() rememberMeStatus: boolean;
  @Input() latestPassword: string;
  @Input() latestEmailAddress: string;
  // Display authentication alert if it is passed into sign in component from parent account container
  @Input() authMessage$: Observable<AuthenticationState>;

  constructor() { }

  ngOnInit() {
  }

  emailChanged(value) {
    // Inform parent container that email has changed
    this.emailHasChanged.emit({'value': value});
  }

  passwordChanged(value) {
    // Inform parent container that password has changed
    this.passwordHasChanged.emit({'value': value});
  }

  submitSignInForm(event) {
    // Method called when the submit action for the sign in form is invoked
    // - Prevent the form for submitting on its own
    event.preventDefault();
    // Inform parent container that the form submit event has occured, but application will be taking over submission
    this.formSubmitHasBegun.emit({'data': event});
  }

  togglePasswordReveal() {
    // Notify parent component of password reveal button click
    this.passwordVisibilityHasBeenToggled.emit({'name': 'PASSWORD VISIBILILTY TOGGLE'});
  }

  toggleRememberMe(event) {
    // Notify parent that the remember me checkbox has changed
    this.rememberMeChecked.emit({'checked': event.target.checked});
  }

}
