import { Component, Output, EventEmitter, Input, OnChanges, HostListener } from '@angular/core';
import { NavNodes } from '../../../models/nav-nodes.interface';
import { getNodeByUid, getNodeByUidAbsolute } from '@hdca/fe-angular-shared';
import { CATEGORIES_NAV_NODE, EVENTS_NAV_NODE } from '@hdca/fe-angular-shared';
@Component({
    selector: 'global-header-flyout',
    templateUrl: './global-header-flyout.component.html'
})
export class GlobalHeaderFlyoutComponent implements OnChanges {
    @Output() flyoutOpen: EventEmitter<any> = new EventEmitter();
    @Output() flyoutClose: EventEmitter<any> = new EventEmitter();
    @Input() navNodes: NavNodes;
    @Input() shopByDepartmentOpen: boolean;
    categoriesNavNodes: NavNodes[];
    eventsNavNode: NavNodes;
    activatedL1uid: string;
    activatedEventuid: string;
    selectedL1Category: NavNodes;
    activatedL1Index: number;
    activatedEventIndex: number;
    keyCodes = {
        ENTER_KEY: 13,
        ARROW_DOWN: 40,
        ARROW_UP: 38,
    };

    // Listen for window keydown so that the appearance of flyouts can be controlled with keyboard
    @HostListener('window:keydown', ['$event', '$event.keyCode'])
    onKeyDown($event: KeyboardEvent, keyCode) {
        const currentActiveNode = this.activatedL1uid ? getNodeByUidAbsolute(this.categoriesNavNodes, this.activatedL1uid) :
            this.activatedEventuid ? getNodeByUid(this.eventsNavNode, this.activatedEventuid) : null;

        if (keyCode === this.keyCodes.ENTER_KEY && this.shopByDepartmentOpen && currentActiveNode) {
            $event.preventDefault();
            if (typeof window !== 'undefined') {
                window.location.href = currentActiveNode.url;
            }
        }

        if (keyCode === this.keyCodes.ARROW_UP && this.shopByDepartmentOpen) {
            $event.preventDefault();
            this.activateNode(-1);
        }

        if (keyCode === this.keyCodes.ARROW_DOWN && this.shopByDepartmentOpen) {
            $event.preventDefault();
            this.activateNode(1);
        }
    }

    ngOnChanges() {
        this.categoriesNavNodes = getNodeByUid(this.navNodes, CATEGORIES_NAV_NODE) && getNodeByUid(this.navNodes, CATEGORIES_NAV_NODE).children;
        this.eventsNavNode = getNodeByUid(this.navNodes, EVENTS_NAV_NODE);
    }

    openFlyout() {
        this.flyoutOpen.emit();
    }

    closeFlyout() {
        this.activatedL1uid = '';
        this.activatedEventIndex = -1;
        this.activatedL1Index = -1;
        this.activatedEventuid = '';
        this.flyoutClose.emit();
    }

    activateNode(increment: number) {
        if (this.activatedL1Index >= 0) {
            const currentIndex = increment + this.activatedL1Index;
            if (currentIndex >= 0 && currentIndex < this.categoriesNavNodes.length) {
                this.activateL1(this.categoriesNavNodes[currentIndex], currentIndex);
            } else {
                if (currentIndex < 0) {
                    if (this.eventsNavNode && this.eventsNavNode && this.eventsNavNode.children.length > 0) {
                        this.activateEvent(this.eventsNavNode.children[this.eventsNavNode.children.length - 1], this.eventsNavNode.children.length - 1);
                    } else {
                        this.activateLastItem();
                    }
                } else {
                    this.activateFirstItem();
                }
            }
        } else {
            const currentIndex = increment + this.activatedEventIndex;
            if (currentIndex >= 0 && currentIndex < this.eventsNavNode.children.length) {
                this.activateEvent(this.eventsNavNode.children[currentIndex], currentIndex);
            } else {
                if (currentIndex < 0) {
                    this.activateLastItem();
                } else {
                    this.activateL1(this.categoriesNavNodes[0], 0);
                }
            }
        }
    }

    activateLastItem() {
        this.activateL1(this.categoriesNavNodes[this.categoriesNavNodes.length - 1], this.categoriesNavNodes.length - 1);
    }

    activateFirstItem() {
        if (this.eventsNavNode && this.eventsNavNode.children && this.eventsNavNode.children.length > 0) {
            this.activateEvent(this.eventsNavNode.children[0], 0);
        } else {
            this.activateL1(this.categoriesNavNodes[0], 0);
        }
    }

    activateL1(node: NavNodes, index: number) {
        this.activatedL1Index = index;
        this.activatedEventIndex = -1;
        this.deactivateEvent();
        this.activatedL1uid = node.uid;
        this.selectedL1Category = node;
    }

    activateEvent(node: NavNodes, index: number) {
        this.activatedEventIndex = index;
        this.activatedL1Index = -1;
        this.activatedEventuid = node.uid;
        this.activatedL1uid = '';
    }

    deactivateEvent() {
        this.activatedEventuid = '';
    }

    categoryActive(node: NavNodes) {
        return node.uid === this.activatedL1uid ? 'hdca-gheader-flyout-shop__l1list-link--is-active' : '';
    }

    eventActive(node: NavNodes) {
        return node.uid === this.activatedEventuid ? 'hdca-gheader-flyout-shop__l1list-link--is-active' : '';
    }
}
