import { Component, Input } from '@angular/core';
// Model Imports
import { SearchKeywordSuggestions, SearchProductCategories } from '../../../models/global-header-search.interface';
// Environment variable imports
import { LanguageService } from '@hdca/fe-angular-shared';

@Component({
  selector: '[global-header-search-autosuggest-list-item]',
  templateUrl: './global-header-search-autosuggest-list-item.component.html'
})
export class GlobalHeaderSearchAutosuggestListItemComponent {
  @Input() item: string;
  @Input() highlightTerm: String;
  @Input() itemSubList: Array<SearchProductCategories>;
  @Input() relevance: boolean;

  // Render the keyword if it is not equal to the search term
  // Render the category sublist for the search term

  buildListItemSublistItemLink(category) {
    // Highlight term in this context is the search keyword (keyword that matches the search term)
    const keyword = this.highlightTerm;
    // Construct the product category link in the item sublist
    // Two arguments:
    // - category: category name
    // - keyword: search keyword, will match the search term (in the input field)
    // Process:
    // 1 - Get the base URL for site
    const baseUrl = typeof window !== 'undefined' && window && window.location && window.location.origin;
    // 2 - Get language for requests
    const lang = this.langService.getLanguage();
    // 3 - construct query string
    const queryStrings = `autoSuggest=prodcat&q=${keyword}:relevance:category:${category}` +
      `#!q=${keyword}:relevance:category:${category}`;
    // 4 - Combine query string with search URL and base URL
    const url = lang === 'fr' ?
      `${baseUrl}/fr/accueil/resultats.products.html?${queryStrings}` :
      `${baseUrl}/en/home/search.products.html?${queryStrings}`;
    return url;
  }

  buildListItemLink() {
    // Construct the keyword link
    // Two arguments:
    // - keyword: the keyword name
    // - relevance: relevant keyword (relevant keywords are those that DO NOT match the search term in input field)
    // 1 - Get the base URL for site
    const baseUrl = window && window.location && window.location.origin;
    // 2 - Get language for requests
    const lang = this.langService.getLanguage();
    // 3 - Get relevance
    const relevance = this.relevance;
    // 4 - get keyword name
    const keyword = this.item;
    // Construct URL based on whether or not relevance or keyword takes precedence
    // - relevance is for keywords beside the keyword matching the search term
    // - keyword: search keyword that matches the search term
    if (relevance) {
      // 3 - construct query string
      const queryStrings = `autoSuggest=${keyword}&q=${relevance}:relevance#!q=${relevance}:relevance`;
      // 4 - Combine query string with search URL and base URL
      return lang === 'fr' ?
        `${baseUrl}/fr/accueil/resultats.html?${queryStrings}` :
        `${baseUrl}/en/home/search.html?${queryStrings}`;
    } else {
      // 3 - construct query string
      const queryStrings = `autoSuggest=${keyword}&q=${keyword}:relevance#!q=${keyword}:relevance`;
      // 4 - Combine query string with search URL and base URL
      return lang === 'fr' ?
        `${baseUrl}/fr/accueil/resultats.html?${queryStrings}` :
        `${baseUrl}/en/home/search.html?${queryStrings}`;
    }
  }

  constructor(
    private langService: LanguageService,
  ) {}

}
