import { Component, Input, OnInit, OnDestroy } from '@angular/core';
// Model Imports
import { SearchKeywordSuggestions, SearchProductCategories } from '../../../models/global-header-search.interface';
// ngrx store imports
import { Store } from '@ngrx/store';
import * as fromApp from '../../../../../reducers';
import { GlobalHeaderSearchState } from '../../../reducers/global-header-search/global-header-search.reducer';
import * as searchActions from '../../../actions/global-header/global-header-search/global-search.action';
import { Subscription } from 'rxjs';
import { CookieService } from 'ngx-cookie';
// Subscription imports
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
// Other app imports
import { LanguageService, StoreService } from '@hdca/fe-angular-shared';

@Component({
  selector: '[global-header-search-autosuggest-list]',
  templateUrl: './global-header-search-autosuggest-list.component.html'
})
export class GlobalHeaderSearchAutosuggestListComponent extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
  @Input() listItems: Array<SearchKeywordSuggestions>;
  @Input() activeKeyword: String;
  @Input() sublist: Array<SearchProductCategories>;
  selectedIndex: Number;

  onKeywordHover(index, value) {
    // Get current store if it exists
    const localizedPOS = this.storeService.getStore();
    const lang = this.langService.getLanguage();
    // When hovering over a keyword, perform a new Top Results query to refresh the top 3 products
    // Update the current hover keyword in state, so it can be compared to the activeKeyword for UI purposes
    this.store.dispatch(new searchActions.SearchKeywordSelectionChanged(index, value, true));
    this.store.dispatch(new searchActions.LoadTopSearchResults('Online', lang, value, localizedPOS, true));
  }

  ngOnInit() {
    // Keep up to date on selected index for keyword list, for updating Top Results
    const autosuggestSub: Subscription = this.store.select(fromApp.getGlobalHeaderSearch).
        subscribe((state: GlobalHeaderSearchState) => {
            this.selectedIndex = state.selectedIndex;
        });
    this.subscriptions.push(autosuggestSub);
  }

  ngOnDestroy() {
      super.ngOnDestroy();
  }

  constructor(
    private store: Store<fromApp.State>,
    private cookieService: CookieService,
    private storeService: StoreService,
    private langService: LanguageService
  ) { super(); }
}
