import { Component, Output, Input, EventEmitter, OnInit, AfterViewInit, OnDestroy } from '@angular/core';
// ngrx imports
import { Store } from '@ngrx/store';
import * as fromApp from '../../../../../reducers';
import * as fromGlobalHeaderOverall from '../../../reducers/global-header-overall/global-header-overall.reducer';
import { Subscription } from 'rxjs';
import { Observable } from 'rxjs';
import { GlobalHeaderLocalizationUI } from '../../../models/global-header-localize.interface';

// Shared app imports
import { LanguageService } from '@hdca/fe-angular-shared';
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';

// Services
import { GeoLocatorService } from '../../../../../services/geo-locator/geo-locator.service';
import { UserInfoService } from '@hdca/fe-angular-shared';
import * as globalHeaderLocalizeActions from '../../../actions/global-header/global-header-localize/global-header-localize.action';
import { DialogService } from '@hdca/fe-angular-dialog';

@Component({
    selector: 'global-header-utility',
    templateUrl: './global-header-utility.component.html'
})
export class GlobalHeaderUtilityComponent extends ManageSubscriptionsContainer implements OnInit, AfterViewInit, OnDestroy {
    // My Account toggle events
    @Output() open: EventEmitter<any> = new EventEmitter();
    @Output() close: EventEmitter<any> = new EventEmitter();
    // UI text
    @Input() userFirstName: string;
    @Input() isSignedIn: boolean;
    @Input() quantity: string;
    // Visibility flag
    visible: boolean;
    cartUrl: string;
    userId: string;
    nearestStoreSelectedUI$: Observable<GlobalHeaderLocalizationUI>;
    toolTipShow = true;
    TOOL_TIP_SHOW_TIME = 5000;
    spinner = false;

    constructor(
        private store: Store<fromApp.State>,
        private langService: LanguageService,
        private user: UserInfoService,
        private geoLocatorService: GeoLocatorService,
        private dialog: DialogService
    ) { super(); }

    // Control the appearance of the My Account feature

    ngOnInit() {
        // Manage the Header UI
        // - Goal: prevent conflicts between the various header components as some components affects the visibility of others
        const globalHeaderUI: Subscription = this.store.select(fromApp.getGlobalHeaderOverall).
            subscribe((state: fromGlobalHeaderOverall.GlobalHeaderOverallState) => {
                this.visible = state.myAccountOpen;
            });
        this.subscriptions.push(globalHeaderUI);

        const translateURLSub: Subscription = this.langService.translateURLAsync('/cart').subscribe(translated => {
            this.cartUrl = `/${translated['cart']}`;
        });
        this.subscriptions.push(translateURLSub);
        // Get user name
        const currentUser = this.user.getUserAccountData();
        this.userId = currentUser.email || 'anonymous';
        this.getUpdates();
    }
    getUpdates() {
        this.nearestStoreSelectedUI$ = this.store.select(state => state.globalHeaderLocalize.nearestStoreSelectedUI);
        this.nearestStoreSelectedUI$.subscribe(stores => {
            this.spinner = false;
        });
    }

    ngAfterViewInit() {
        if (typeof window !== 'undefined') {
            setTimeout(() => {
                this.toolTipShow = false;
            }, this.TOOL_TIP_SHOW_TIME);
        }
    }

    ngOnDestroy() {
        super.ngOnDestroy();
    }

    myAccountClick() {
        // Toggles the my account flyout open or closed
        if (!this.visible) {
            this.open.emit({'name': 'MY ACCOUNT', 'open': true});
        } else {
            this.close.emit({'name': 'MY ACCOUNT', 'open': false});
        }
    }
    findMyNearestStoreClick() {
        // GET CURRENT LOCATION USING BROWSER
        this.spinner = true;

        const locateSub: Subscription = this.geoLocatorService.getLocation().subscribe(
            rep => {
                if (typeof window !== 'undefined') {
                    setTimeout(() => {
                        const params = {
                            pageSize: '1',
                            currentPage: '0',
                            latitude: rep.coords.latitude,
                            longitude: rep.coords.longitude
                        };

                        this.store.dispatch(new globalHeaderLocalizeActions.LoadAutoLocalize(this.langService.getLanguage(), params, this.userId, false));
                        this.spinner = false;
                        this.closeSelectMyStoreModal();
                    }, 2000);
                }
            }
        );
        this.subscriptions.push(locateSub);
    }
    closeSelectMyStoreModal() {
        this.dialog.close('dialog-store-selector');
    }
}
