import { Component, Input, OnInit, ViewChild, OnDestroy } from '@angular/core';
import { GlobalHeaderLocalizeStoreSelector } from '../../models/global-header-localize.interface';
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
import { Subscription } from 'rxjs';
declare var Microsoft: any;

@Component({
    selector: 'store-selector-bing',
    templateUrl: './store-selector-bing.component.html'
})
export class StoreSelectorBingComponent extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
    private map;
    @Input('stores') stores;
    @Input('fsa') fsa;
    @ViewChild('bingMap') bingMap;
    constructor() {
        super();
    }

    ngOnInit() {
        if (typeof Microsoft !== 'undefined') {
            this.loadMap();
        }
        this.pushSubscriptions();
    }

    ngOnDestroy() {
        super.ngOnDestroy();
    }

    loadMap() {
        this.map = Microsoft && new Microsoft.Maps.Map(this.bingMap.nativeElement, {
            credentials: 'AvroW_HXR0kSPh2sMToqCbu4eOHaDgjxdsdtqQO4QntbWg9ituT9YjwRbWEewkms',
        });
    }

    pushSubscriptions() {
        const storeSub: Subscription = this.stores.subscribe({
            next: state => this.renderMap(state)
        });
        this.subscriptions.push(storeSub);
    }

    renderMap(state: GlobalHeaderLocalizeStoreSelector) {
        if (state.stores !== null) {
            this.map.entities.clear();
            const locationArr = [];
            let pin,
                location;
            let i = 0;
            for (const store of state.stores) {
                i++;
                location = new Microsoft.Maps.Location(store.geoPoint.latitude, store.geoPoint.longitude);
                pin = new Microsoft.Maps.Pushpin(location, { text: `${i}` });
                this.map.entities.push(pin);
                locationArr.push(location);
            }
            const view = Microsoft.Maps.LocationRect.fromLocations(locationArr);
            this.map.setView({
                bounds: view
            });
        }
    }
}
