// core
import { Component, OnChanges, Input } from '@angular/core';

// services
import { LanguageService } from '@hdca/fe-angular-shared';
import { getNodeByUidAbsolute } from '@hdca/fe-angular-shared';

// models
import { MessageLink, NavNodes } from '../../../models/nav-nodes.interface';

@Component({
    selector: 'global-header-scarf',
    templateUrl: './global-header-scarf.container.html'
})
export class GlobalHeaderScarfContainer implements OnChanges {

    /******************************************
     // THIS CONTAINER CONTAINS THE DATA, LOGIC AND VIEW
     // FOR THE GLOBAL HEADER SCARF.
     // IT FIRST CHECKS TO SEE IF WE HAVE SNAPSHOT
     // DATA, IF NONE EXISTS, IT GETS THE DATA
     // FROM THE NAV NODE END POINT
     ******************************************/

    @Input() navNodes: NavNodes;
    scarf: MessageLink;
    language: string;
    navNodesString: string;
    uidString: string;

    constructor(
        private langService: LanguageService) {
        this.scarf = null;
    }

    /******************************************
     // CORE
     ******************************************/

    ngOnChanges() {
        this.navNodesString = 'navNodes';
        this.uidString = 'GlobalMessageNavNode';
        this.language = this.langService.getLanguage();

        this.getSnapshot();
    }

    /******************************************
     // SETUP
     ******************************************/

    getSnapshot() {
        // only assign to scarf is navNodes is not null
        if (this.navNodes !== undefined) {
            this.scarf = this.filterNavNodes(this.navNodes);
        }
    }

    /******************************************
     // UTIL
     ******************************************/

    private filterNavNodes(navNodes) {
        const globalMessageNavNode = getNodeByUidAbsolute(navNodes && navNodes.children, this.uidString);
        return globalMessageNavNode && globalMessageNavNode.messageLinks && globalMessageNavNode.messageLinks[0];
    }
}
