// Angular Imports
import { Component, OnInit, ChangeDetectorRef, OnDestroy, Input } from '@angular/core';
// Model Imports
import { SearchAutosuggestDetails, SearchKeywordSuggestions } from '../../../models/global-header-search.interface';
// State imports
import { Subscription } from 'rxjs/Subscription';
import * as fromApp from '../../../../../reducers';
import * as fromGlobalHeaderSearch from '../../../reducers/global-header-search/global-header-search.reducer';
import { Store } from '@ngrx/store';
// Subscription imports
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
// Other app imports
import { HideAutoSuggest } from '../../../actions/global-header/global-header-search/global-search.action';
@Component({
  selector: 'global-header-search-autosuggest',
  templateUrl: './global-header-search-autosuggest.container.html'
})
export class GlobalHeaderSearchAutosuggestContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy {
  autosuggestDetails: SearchAutosuggestDetails;
  keywordsuggestions: SearchKeywordSuggestions;
  searchTerm: String;
  keyword: String;
  @Input() originalSearch: string;
  @Input()   showDetails: boolean;
  // Stencils for keywords and top results, placeholders that appear during loading states
  keywordStencil: Array<string>;
  topResultsStencil: Array<string>;

  ngOnInit() {
    // Assign stencil for keywords and top results to component, consist of arrays with placeholder content
    this.keywordStencil = Array(5).fill('.');
    this.topResultsStencil = Array(3).fill('.');
    // Dto consist of two main elements:
    // - Autosuggest details
    // - Keyword suggestions
    // Autosuggest details object consists of two main elements:
    // - Product categories: two closest matching categories that appear as sublinks
    // - Top product searches
    // Keyword suggestions consists of list of strings, keywords that are the closest match to the current search term
    // Subscribe to the GlobalHeaderSearchState and keep two search autosuggest data elements updated
    const autosuggestSub: Subscription = this.store.select(fromApp.getGlobalHeaderSearch).
        subscribe((state: fromGlobalHeaderSearch.GlobalHeaderSearchState) => {
            this.autosuggestDetails = state.autosuggestDetails;
            this.keywordsuggestions = state.keywordsuggestions;
            this.searchTerm         = state.searchTerm;
            this.keyword            = state.keyword;
            if (!this.ref['destroyed']) {
                this.ref.detectChanges();
            }
        });
    this.subscriptions.push(autosuggestSub);
  }

    ngOnDestroy() {
        super.ngOnDestroy();
    }

    constructor(private store: Store<fromApp.State>,
      private ref: ChangeDetectorRef) {
      super();
    }

    searchItemClicked(bool) {
      this.store.dispatch(new HideAutoSuggest(bool));
    }
}
