import { Component, EventEmitter, Input, OnInit, Output, OnDestroy, OnChanges } from '@angular/core';

// import bing maps
import { BingMapService } from '../../../../services/bing-maps/bing-maps.service';
import { Observable } from 'rxjs/Observable';
import * as fromApp from '../../../../reducers';
import { Store } from '@ngrx/store';
import { GlobalHeaderLocalizeStoreSelector, GlobalHeaderLocalizationUI } from '../../models/global-header-localize.interface';
import { CookieService } from 'ngx-cookie';
import { StoreConstants } from '@hdca/fe-angular-shared';
import { Stores } from '../../models/global-header-localize.interface';
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
import { Subscription } from 'rxjs';

@Component({
    selector: 'store-selector',
    templateUrl: './store-selector.container.html'
})
export class StoreSelectorContainer extends ManageSubscriptionsContainer implements OnInit, OnDestroy, OnChanges {
    @Input('stores') stores;
    @Input() presetDefaultQuery?: string;
    @Input() language: string;
    @Output('storeSelectorSearchClick') storeSelectorSearchClick = new EventEmitter();
    @Output('storeSelectorNextClick') storeSelectorNextClick = new EventEmitter();
    @Output('storeSelectorPreviousClick') storeSelectorPreviousClick = new EventEmitter();
    @Output('storeSelectorSelectClick') storeSelectorSelectClick = new EventEmitter();
    @Output('autoLocateClick') autoLocateClick = new EventEmitter();
    @Output('closeSelectMyStoreModal') closeSelectMyStoreModal = new EventEmitter();
    @Output() closeButtonClickEvent: EventEmitter<any> = new EventEmitter();
    mapReady = false;
    storeSelector$: Observable<GlobalHeaderLocalizeStoreSelector>;
    storeSelectorState: GlobalHeaderLocalizeStoreSelector;
    selectedId: string;
    nearestStoreSelectedUI$: Observable<GlobalHeaderLocalizationUI>;
    spinner = false;
    viewToggle = true;

    constructor(private store: Store<fromApp.State>, private cookieService: CookieService) {
        super();
        BingMapService.load()
            .then(res => {
                this.mapReady = true;
            });
    }

    ngOnInit() {
        this.selectedStoreId();
        this.getStores();
        this.pushSubscriptions();
    }

    pushSubscriptions() {
        const localizeSub: Subscription = this.store.select(state => state.globalHeaderLocalize).subscribe({
            next: state => {
                if (this.selectedId !== state.storeId) {
                    this.selectedId = state.storeId;
                }
            }
        });
        this.subscriptions.push(localizeSub);
    }

    ngOnChanges() {
        this.selectedStoreId();
    }

    selectedStoreId() {
        this.selectedId = typeof document !== 'undefined' ? this.cookieService.get(StoreConstants.STORE_KEY) : null;
    }

    getStores() {
        this.storeSelector$ = this.store.select(state => state.globalHeaderLocalize.storeSelector);
        this.nearestStoreSelectedUI$ = this.store.select(state => state.globalHeaderLocalize.nearestStoreSelectedUI);
        this.nearestStoreSelectedUI$.subscribe(stores => {
            this.spinner = false;
            this.closeSelectMyStoreModal.emit();
            this.selectedId = typeof document !== 'undefined' ? this.cookieService.get(StoreConstants.STORE_KEY) : null;
        });
        this.nearestStoreSelectedUI$ = this.store.select(state => state.globalHeaderLocalize.nearestStoreSelectedUI);
        this.nearestStoreSelectedUI$.subscribe(stores => {
            this.spinner = false;
            this.closeSelectMyStoreModal.emit();
            this.selectedId = typeof document !== 'undefined' ? this.cookieService.get(StoreConstants.STORE_KEY) : null;
        });
    }

    onNextClick() {
        this.storeSelectorNextClick.emit();
    }

    onPreviousClick() {
        this.storeSelectorPreviousClick.emit();
    }

    onSearchClick(postal) {
        this.storeSelectorSearchClick.emit(postal);
    }

    onSelectClick(storeId) {
        this.storeSelectorSelectClick.emit(storeId);
    }

    getDirection(store: Stores) {
        const bingAddress = 'https://www.bing.com/maps?rtp=~pos.' + store.geoPoint.latitude + '_' + store.geoPoint.longitude + '_' + store.address.formattedAddress;
        if (typeof window !== 'undefined') {
            window.open(bingAddress, '_blank');
        }
    }

    weeklyFlyer(store: Stores) {
        let weeklyFlyerURL = '/en/home/value-centre/weekly-flyer.html?postal=' + store.address.postalCode;
        if (this.language === 'fr') {
            weeklyFlyerURL = '/fr/accueil/centre-de-valeur/circulaire-hebdomadaire.html?postal=' + store.address.postalCode;
        }
        if (typeof window !== 'undefined') {
            window.open(weeklyFlyerURL, '_blank');
        }
    }

    onFindMyNearestStoreClick() {
        this.autoLocateClick.emit();
    }

    toggleView() {
        this.viewToggle = !this.viewToggle;
    }

    ngOnDestroy() {
        super.ngOnDestroy();
    }
}
