import { Directive, HostListener, OnDestroy } from '@angular/core';
// ngrx store imports
import { Store } from '@ngrx/store';
import * as fromApp from '../../../../reducers';
import * as globalHeaderActions from '../../actions/global-header/global-header-overall/global-header-overall.action';
// App imports
import { ManageSubscriptionsContainer } from '@hdca/fe-angular-shared';
// Other app imports

@Directive({
    selector: '[global-header-interaction]',
})
export class GlobalHeaderDirective extends ManageSubscriptionsContainer implements OnDestroy {
    // Directive for listening to keyboard events while the flyouts are open
    // One scenario
    // - Pressing escape to initiate a search

    // Listen for window keydown so that the appearance of flyouts can be controlled with keyboard
    @HostListener('window:keydown', ['$event', '$event.keyCode'])
    onKeyDown($event: KeyboardEvent, keyCode) {

        const keyCodes = {
            ENTER_KEY: 13,
            ESCAPE: 27,
            ARROW_DOWN: 40,
            ARROW_UP: 38,
        };

        // Use escape key to close flyouts if open
        if (keyCode === keyCodes.ESCAPE) {
            this.hideAllFlyouts();
        }

    }

    hideAllFlyouts() {
        // Hide the Shop By Department or My Account flyout if the escape key is pressed
        this.store.dispatch(new globalHeaderActions.CloseCategoriesAndAccount());
    }

    ngOnDestroy() {
        super.ngOnDestroy();
    }

    constructor(
        private store: Store<fromApp.State>,
    ) { super(); }

}
