// Core import
import { Injectable } from '@angular/core';
// ngrx imports
import { Effect, Actions } from '@ngrx/effects';
import { Action } from '@ngrx/store';
import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';
// App imports
import { MyAccountLoginPortal } from '../../../../services/data-sender/my-account-login.service';
import * as myAccountActions from '../../actions/global-header/global-header-account/global-header-account.action';
import { apiUrls } from '../../../../constants/api.urls';
import { AuthenticationResponseConstants } from '@hdca/fe-angular-shared';
// rxjs additional operators
import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/mergeMap';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/skip';
import 'rxjs/add/operator/takeUntil';
import 'rxjs/add/operator/do';

import { OriginService } from '../../../../services/origin/origin.service';

@Injectable()
export class MyAccountEffects {
    @Effect()
    accountLogin$: Observable<Action> = this.actions$
        .ofType<myAccountActions.InitiateLoginProcess>(myAccountActions.INITIATE_LOGIN_PROCESS)
        .switchMap((action) => {
            return this.postService
                .post(apiUrls.BASE_ACCOUNT_URL(this.originService.getOrigin()) + apiUrls.ACCOUNT_LOGIN(action.lang), action.payload, action.headers, action.lang)
                .mergeMap((response: string) => {
                    // Do response logic here to handle empty code for successful authentication
                    // Pass an easy to interpret code to the login response action
                    let code = '';
                    let successFlag = false;
                    if (!response || response === '' || response.indexOf('null') >= 0) {
                        code = AuthenticationResponseConstants.ACCEPTED;
                        successFlag = true;
                    } else if (response.indexOf('OAUTH213') >= 0 || response.indexOf('OAUTH214') >= 0) {
                        code = AuthenticationResponseConstants.AUTHENTICATION_FAILED;
                    } else if (response.indexOf('OAUTH250') >= 0) {
                        code = AuthenticationResponseConstants.AUTHENTICATION_LOCKED;
                    } else if (response.indexOf('ACC400') >= 0) {
                        code = AuthenticationResponseConstants.RESET_PASSWORD;
                    } else {
                        code = AuthenticationResponseConstants.AUTHENTICATION_ERROR;
                    }

                    return [
                        new myAccountActions.LoginResponse(code, successFlag)
                    ];
                })
                .catch(err => of(new myAccountActions.LoginError(err)));
        });

    @Effect()
    accountLogout$: Observable<Action> = this.actions$
        .ofType<myAccountActions.InitiateLogoutProcess>(myAccountActions.INITIATE_LOGOUT_PROCESS)
        .switchMap((action) => {
            return this.postService
                .post(apiUrls.BASE_ACCOUNT_URL(this.originService.getOrigin()) + apiUrls.ACCOUNT_LOGOUT(action.lang), {observe: 'response'}, action.headers, action.lang)
                .do((response: any) => {
                    // If logout is successful, reload the page
                    if (typeof window !== 'undefined') {
                        window.location.reload(true);
                    }
                })
                .catch(err => {
                    if (typeof window !== 'undefined') {
                        window.location.reload(true);
                    }
                    return of(new myAccountActions.LogoutError());
                });
            });

    constructor(
        private actions$: Actions,
        private postService: MyAccountLoginPortal,
        private originService: OriginService
    ) { }
}
