
import { Injectable } from '@angular/core';
import { Effect, Actions } from '@ngrx/effects';
import { Action } from '@ngrx/store';
import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';
import { DataFetcher } from '../../../../services/data-fetcher/data-fetcher.service';
import { ToggleApiError } from '../../../../actions/errors';
import * as navNodesActions from '../../actions/global-header/nav-nodes/nav-nodes.action';
import { NavNodes } from '../../models/nav-nodes.interface';
import { apiUrls } from '../../../../constants/api.urls';

import 'rxjs/add/operator/catch';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/switchMap';

import { OriginService } from '../../../../services/origin/origin.service';

const errorUrl = null;

@Injectable()
export class NavNodesEffect {
    @Effect()
    loadNavNodes$: Observable<Action> = this.actions$
        .ofType<navNodesActions.LoadNavNodes>(navNodesActions.LOAD_NAV_NODES)
        .switchMap((action) => {
            return this.service
                .fetch(apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_NAV_NODES(), action.lang)
                .map((navNodes: NavNodes) => new navNodesActions.LoadNavNodesSuccess(navNodes, true))
                .catch(err => of(new ToggleApiError(err, errorUrl,
                    (apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_NAV_NODES(), action.lang))));
        });

    constructor(
        private actions$: Actions,
        private service: DataFetcher,
        private originService: OriginService
    ) { }
}
