// directives
import { GlobalSearchDirective } from './directives/global-header/global-header-search.directive';
import { GlobalHeaderDirective } from './directives/global-header/global-header.directive';

// containers
import { GlobalHeaderContainer } from './containers/global-header/global-header.container';
import { GlobalHeaderScarfContainer } from './containers/global-header/global-header-scarf/global-header-scarf.container';
import { GlobalHeaderAccountContainer } from './containers/global-header/global-header-account/global-header-account.container';
import { GlobalHeaderLocalizationContainer } from './containers/global-header/global-header-localization/global-header-localization.container';
import { GlobalHeaderSearchContainer } from './containers/global-header/global-header-search/global-header-search.container';
import { GlobalHeaderSearchInputFieldContainer } from './containers/global-header/global-header-search-input-field/global-header-search-input-field.container';
import { GlobalHeaderSearchAutosuggestContainer } from './containers/global-header/global-header-search-autosuggest/global-header-search-autosuggest.container';
import { StoreSelectorContainer } from './containers/store-selector/store-selector.container';
import { StoreSelectorWrapperContainer } from './containers/store-selector-wrapper/store-selector-wrapper.container';
import { StoreSelectorBingComponent } from './components/store-selector-bing/store-selector-bing.component';

// components

import { GlobalHeaderCategoriesComponent } from './components/global-header/global-header-categories/global-header-categories.component';
import { GlobalHeaderLogoComponent } from './components/global-header/global-header-logo/global-header-logo.component';
import { GlobalHeaderNavComponent } from './components/global-header/global-header-nav/global-header-nav.component';
import { GlobalHeaderUtilityComponent } from './components/global-header/global-header-utility/global-header-utility.component';
import { GlobalHeaderAccountCreateComponent } from './components/global-header/global-header-account-create/global-header-account-create.component';
import { GlobalHeaderAccountSignInComponent } from './components/global-header/global-header-account-sign-in/global-header-account-sign-in.component';
import { GlobalHeaderAccountStatusComponent } from './components/global-header/global-header-account-status/global-header-account-status.component';
import { GlobalHeaderFlyoutComponent } from './components/global-header/global-header-flyout/global-header-flyout.component';
import { GlobalHeaderLocalizationAkamaiComponent } from './components/global-header/global-header-localization-akamai/global-header-localization-akamai.component';
import { GlobalHeaderLocalizationHoursComponent } from './components/global-header/global-header-localization-hours/global-header-localization-hours.component';
import { GlobalHeaderMobileComponent } from './components/global-header/global-header-mobile/global-header-mobile.component';
import { GlobalHeaderSearchAutosuggestDetailsComponent } from './components/global-header/global-header-search-autosuggest-details/global-header-search-autosuggest-details.component';
import { GlobalHeaderSearchAutosuggestDetailsItemComponent } from './components/global-header/global-header-search-autosuggest-details-item/global-header-search-autosuggest-details-item.component';
import { GlobalHeaderSearchAutosuggestListComponent } from './components/global-header/global-header-search-autosuggest-list/global-header-search-autosuggest-list.component';
import { GlobalHeaderSearchAutosuggestListItemComponent } from './components/global-header/global-header-search-autosuggest-list-item/global-header-search-autosuggest-list-item.component';
import { GlobalHeaderFlyoutContentComponent } from './components/global-header/global-header-flyout/global-header-flyout-content/global-header-flyout-content.component';
import { GlobalHeaderFlyoutContentSmallComponent } from './components/global-header/global-header-flyout/global-header-flyout-content-small/global-header-flyout-content-small.component';
import { GlobalHeaderFlyoutSmallComponent } from './components/global-header/global-header-flyout/global-header-flyout-small/global-header-flyout-small.component';

import { HttpClient } from '@angular/common/http';
import { TranslatesBrowserLoaderService } from '../translation/translation-browser/translation-browser-loader.service';
import { TransferState } from '@angular/platform-browser';

// effects
import { GlobalHeaderLocalizeEffect } from './effects/global-header-localize/global-header-localize.effect';
import { NavNodesEffect } from './effects/nav-nodes/nav-nodes.effect';
import { GlobalSearchEffects } from './effects/global-search/global-search.effect';
import { MyAccountEffects } from './effects/global-header-my-account/global-header-my-account.effect';

// modules
import { EffectsModule } from '@ngrx/effects';
import { CommonModule } from '@angular/common';
import { NgModule, ModuleWithProviders } from '@angular/core';
import { HDCASharedModule } from '@hdca/fe-angular-shared';
import { HDCADialogModule } from '@hdca/fe-angular-dialog';
import { RouterModule } from '@angular/router';

// NgxTranslate
import { TranslateLoader, TranslateModule } from '@ngx-translate/core';

export function translateStaticLoaderGlobalHeader(http: HttpClient, transferState: TransferState): TranslatesBrowserLoaderService {
    return new TranslatesBrowserLoaderService('/node-assets/i18n/global-header/', '.json', transferState, http);
}

@NgModule({
    imports: [
        CommonModule,
        HDCADialogModule,
        HDCASharedModule,
        TranslateModule.forChild({
            loader: {
                provide: TranslateLoader,
                useFactory: translateStaticLoaderGlobalHeader,
                deps: [HttpClient, TransferState]
            }
        }),
        EffectsModule.forFeature([
            GlobalSearchEffects,
            MyAccountEffects,
            NavNodesEffect,
            GlobalHeaderLocalizeEffect,
        ]),
        RouterModule.forChild([])
    ],
    declarations: [
        GlobalHeaderContainer,
        GlobalHeaderScarfContainer,
        GlobalHeaderAccountContainer,
        GlobalHeaderSearchContainer,
        GlobalHeaderSearchInputFieldContainer,
        GlobalHeaderSearchAutosuggestContainer,
        GlobalHeaderLocalizationContainer,
        GlobalHeaderCategoriesComponent,
        GlobalHeaderNavComponent,
        GlobalHeaderLogoComponent,
        GlobalHeaderUtilityComponent,
        GlobalHeaderAccountSignInComponent,
        GlobalHeaderAccountCreateComponent,
        GlobalHeaderAccountStatusComponent,
        GlobalHeaderFlyoutComponent,
        GlobalHeaderLocalizationAkamaiComponent,
        GlobalHeaderLocalizationHoursComponent,
        GlobalHeaderMobileComponent,
        GlobalHeaderSearchAutosuggestDetailsComponent,
        GlobalHeaderSearchAutosuggestDetailsItemComponent,
        GlobalHeaderSearchAutosuggestListComponent,
        GlobalHeaderSearchAutosuggestListItemComponent,
        GlobalHeaderFlyoutContentComponent,
        GlobalHeaderFlyoutSmallComponent,
        GlobalHeaderFlyoutContentSmallComponent,
        GlobalSearchDirective,
        GlobalHeaderDirective,
        StoreSelectorContainer,
        StoreSelectorWrapperContainer,
        StoreSelectorBingComponent
    ],
    exports: [
        GlobalHeaderContainer
    ]
})
export class HDCAGlobalHeaderModule {
    constructor() {
    }

    static forRoot(): ModuleWithProviders {
        return {
          ngModule: HDCAGlobalHeaderModule,
          providers: [
            ]
        };
      }

}
