import * as globalHeaderLocalize from '../../actions/global-header/global-header-localize/global-header-localize.action';

// models
import { GlobalHeaderLocalizeAkamai, GlobalHeaderLocalizeHours, GlobalHeaderLocalizeStoreSelector, GlobalHeaderLocalizeUI, GlobalHeaderLocalizationUI } from '../../models/global-header-localize.interface';

export interface GlobalHeaderLocalize {
    localized?: boolean;
    storeHours?: string;
    storeTitle?: string;
    storeId?: string;
    pickUpStoreId?: string;
    latitude?: number;
    longitude?: number;
    ui?: GlobalHeaderLocalizeUI;
    akamai?: GlobalHeaderLocalizeAkamai;
    hours?: GlobalHeaderLocalizeHours;
    storeSelector?: GlobalHeaderLocalizeStoreSelector;
    nearestStoreSelectedUI?: GlobalHeaderLocalizationUI;
}

export const nullState: GlobalHeaderLocalize = {
    localized: undefined,
    storeHours: '',
    storeTitle: '',
    storeId: undefined,
    pickUpStoreId: undefined,
    latitude: null,
    longitude: null,
    ui: {
        akamai: false,
        hours: false,
        storeSelector: false
    },
    akamai: {
        storeTitle: '',
        storeHours: '',
        storeAddress: '',
        storePhone: '',
        autoLocateLoading: false
    },
    hours: {
        formattedHours: null,
        phone: ''
    },
    storeSelector: {
        stores: null,
        pagination: null,
        query: ''
    },
    nearestStoreSelectedUI: {
        nearestStoreSelected: false,
    }
};

export const initialState = ((): GlobalHeaderLocalize => {
    return nullState;
})();

export function globalHeaderLocalizeReducer(state = initialState, action: globalHeaderLocalize.GlobalHeaderLocalizeActions): GlobalHeaderLocalize {
    switch (action.type) {

        /******************************************
         // REACTIONS
         ******************************************/

        case globalHeaderLocalize.UPDATE_GLOBAL_HEADER_LOCALIZE: {
            return { ...state, ...action.newState };
        }

        case globalHeaderLocalize.UPDATE_GLOBAL_HEADER_LOCALIZE_AKAMAI: {
            return {
                ...state,
                akamai: {
                    ...state.akamai,
                    ...action.newState
                }
            };
        }

        case globalHeaderLocalize.UPDATE_GLOBAL_HEADER_LOCALIZE_HOURS: {
            return {
                ...state,
                hours: {
                    ...state.hours,
                    ...action.newState
                }
            };
        }

        case globalHeaderLocalize.UPDATE_GLOBAL_HEADER_LOCALIZE_STORE_SELECTOR: {
            return {
                ...state,
                storeSelector: {
                    ...state.storeSelector,
                    ...action.newState
                }
            };
        }

        case globalHeaderLocalize.TOGGLE_GLOBAL_HEADER_LOCALIZE_UI: {
            return {
                ...state,
                ui: {
                    ...state.ui,
                    ...action.newState
                }
            };
        }

        case globalHeaderLocalize.UPDATE_GLOBAL_HEADER_LOCALIZATION_UI: {
            return {
                ...state,
                nearestStoreSelectedUI: {
                    ...state.nearestStoreSelectedUI,
                    ...action.newState
                }
            };
        }

        case globalHeaderLocalize.UNLOCALIZE: {
            return {
                ...nullState
            };
        }

        case globalHeaderLocalize.PICK_UP_STORE_ID_RECEIVED:
            return {
                ...state,
                pickUpStoreId: action.pickUpStoreId
            };

        case globalHeaderLocalize.STORE_ID_RECEIVED:
            return {
                ...state,
                storeId: action.storeId
            };

        default: {
            return state;
        }
    }
}
