// Import actions
import * as globalHeaderOverallActions from '../../actions/global-header/global-header-overall/global-header-overall.action';

export interface GlobalHeaderOverallState {
  storeLocatorOpen: boolean;
  autoSuggestOpen: boolean;
  shopByDepartmentOpen: boolean;
  storeInfoOpen: boolean;
  myAccountOpen: boolean;
  cartQuantity: string;
}

export const nullState: GlobalHeaderOverallState = {
  storeLocatorOpen: false,
  autoSuggestOpen: false,
  shopByDepartmentOpen: false,
  storeInfoOpen: false,
  myAccountOpen: false,
  cartQuantity: '0'
};

export const initialState = ((): GlobalHeaderOverallState => {
  return nullState;
})();

export function globalHeaderOverallReducer(state = initialState, action: globalHeaderOverallActions.GlobalHeaderOverallActions): GlobalHeaderOverallState {
  switch (action.type) {
    case globalHeaderOverallActions.GLOBAL_HEADER_MY_ACCOUNT_OPEN: {
      // Shop by Department menu cannot be open at same time as My Account (mobile and desktop)
      return {
        ...state,
        myAccountOpen: true,
        shopByDepartmentOpen: false
      };
    }

    case globalHeaderOverallActions.GLOBAL_HEADER_MY_ACCOUNT_CLOSE: {
      return {
        ...state,
        myAccountOpen: false
      };
    }

    case globalHeaderOverallActions.GLOBAL_HEADER_CLOSE_CATEGORIES_ACCOUNT: {
      // Pressing escape key when either categories or account is open causes both to close
      return {
        ...state,
        myAccountOpen: false,
        shopByDepartmentOpen: false
      };
    }

    case globalHeaderOverallActions.GLOBAL_HEADER_CART_COUNT: {
      // Update the cart quantity
      return {
        ...state,
        cartQuantity: action.payload
      };
    }

    case globalHeaderOverallActions.GLOBAL_HEADER_FLYOUT_OPEN: {
      // Shop by Department menu cannot be open at same time as My Account (mobile and desktop)
      return {
        ...state,
        myAccountOpen: false,
        shopByDepartmentOpen: true
      };
    }

    case globalHeaderOverallActions.GLOBAL_HEADER_FLYOUT_CLOSE: {
      return {
        ...state,
        shopByDepartmentOpen: false
      };
    }

    default: {
      return state;
    }
  }
}
