import { Inject, NgModule, PLATFORM_ID, ModuleWithProviders } from '@angular/core';
import { CommonModule } from '@angular/common';
import { StoreModule } from '@ngrx/store';
import { EffectsModule } from '@ngrx/effects';
import { LanguageService } from '@hdca/fe-angular-shared';
import { PETADestinationsRequestEffect } from './effects/peta-destinations.effect';
import { PETARequestEffect } from './effects/peta.effect';
import { petaReducers } from './reducers';
import { PersonalizedETAContainer } from './containers/peta.container';

@NgModule({
  imports: [
    CommonModule,
    StoreModule.forFeature('peta', petaReducers),
    EffectsModule.forFeature([
      PETADestinationsRequestEffect,
      PETARequestEffect,
    ]),
  ],
  providers: [],
  declarations: [
    PersonalizedETAContainer
  ],
  exports: [
    PersonalizedETAContainer
  ],
})
/**
 * description - This core module contains the code necessary for peta related functions.
 */
export class HDCAPetaModule {
  constructor(private languageService: LanguageService, @Inject(PLATFORM_ID) platformId: any) {
    const lang = this.languageService.getLanguage();
    this.languageService.setLanguage(lang);
  }

  static forRoot(): ModuleWithProviders {
    return {
      ngModule: HDCAPetaModule
    };
  }
}
