import {
  ActionReducer,
  ActionReducerMap,
  MetaReducer
} from '@ngrx/store';
import { production } from '../tokens';
import { storeFreeze } from 'ngrx-store-freeze';
import { localStorageSync } from 'ngrx-store-localstorage';

import { ErrorState, ErrorsReducer} from './errors.reducer';
import * as fromModal from './modal.reducer';
import * as fromGlobalHeaderOverall from '../modules/global-header/reducers/global-header-overall/global-header-overall.reducer';
import * as fromGlobalHeaderAccount from '../modules/global-header/reducers/global-header-account/global-header-account.reducer';
import * as fromGlobalHeaderLocalize from '../modules/global-header/reducers/global-header-localize/global-header-localize.reducer';
import * as fromGlobalRouting from '../modules/global-header/reducers/global-routing/global-routing.reducer';
import * as fromGlobalHeaderSearch from '../modules/global-header/reducers/global-header-search/global-header-search.reducer';
import * as fromNavNodes from '../modules/global-header/reducers/nav-nodes/nav-nodes.reducer';
import { EventTransceiverState, eventTransceiverReducer } from '../modules/event-transceiver/reducers/event-transceiver.reducer';

// add in any global state items we want here
export interface State {
isError: ErrorState;
modal: fromModal.ModalState;
globalHeaderOverall: fromGlobalHeaderOverall.GlobalHeaderOverallState;
globalHeaderAccount: fromGlobalHeaderAccount.GlobalHeaderAccountState;
globalHeaderLocalize: fromGlobalHeaderLocalize.GlobalHeaderLocalize;
globalRouting: fromGlobalRouting.GlobalRouting;
globalHeaderSearch: fromGlobalHeaderSearch.GlobalHeaderSearchState;
navNodes: fromNavNodes.NavNodesState;
eventTransceiver: EventTransceiverState;
}

export const reducers: ActionReducerMap<State> = {
isError: ErrorsReducer,
modal: fromModal.ModalReducer,
globalHeaderOverall: fromGlobalHeaderOverall.globalHeaderOverallReducer,
globalHeaderAccount: fromGlobalHeaderAccount.globalHeaderAccountReducer,
globalHeaderLocalize: fromGlobalHeaderLocalize.globalHeaderLocalizeReducer,
globalRouting: fromGlobalRouting.globalRoutingReducer,
globalHeaderSearch: fromGlobalHeaderSearch.globalHeaderSearchReducer,
navNodes: fromNavNodes.navNodesReducer,
eventTransceiver: eventTransceiverReducer
};

export function localStorageSyncReducer(reducer: ActionReducer<any>): ActionReducer<any> {
if (typeof window !== 'undefined') {
  return localStorageSync({ keys: ['serviceConnect'] })(reducer);
} else {
  return reducer;
}
}

export const metaReducers: MetaReducer<State>[] = !production
? [storeFreeze, localStorageSyncReducer]
: [localStorageSyncReducer];

export const getAppError = (state: State): ErrorState => state.isError;
export const getGlobalHeaderOverall = (state: State): fromGlobalHeaderOverall.GlobalHeaderOverallState => state.globalHeaderOverall;
export const getGlobalHeaderAccount = (state: State): fromGlobalHeaderAccount.GlobalHeaderAccountState => state.globalHeaderAccount;
export const getGlobalHeaderLocalize = (state: State): fromGlobalHeaderLocalize.GlobalHeaderLocalize => state.globalHeaderLocalize;
export const getGlobalRouting = (state: State): fromGlobalRouting.GlobalRouting => state.globalRouting;
export const getGlobalHeaderSearch = (state: State): fromGlobalHeaderSearch.GlobalHeaderSearchState => state.globalHeaderSearch;
export const getNavNodes = (state: State): fromNavNodes.NavNodesState => state.navNodes;
export const getEventTransceiver = (state: State): EventTransceiverState => state.eventTransceiver;
