import { Inject, Injectable, PLATFORM_ID } from '@angular/core';
import { Resolve } from '@angular/router';
import { LanguageService } from '@hdca/fe-angular-shared';
import { NavNodes } from '../modules/global-header/models/nav-nodes.interface';
import { ActivatedRouteSnapshot, RouterStateSnapshot } from '@angular/router';
import { DataFetcher } from '../services/data-fetcher/data-fetcher.service';
import { apiUrls } from '../constants/api.urls';
import 'rxjs/add/operator/first';
import { Observable, of, race } from 'rxjs';
import { OriginService } from '../services/origin/origin.service';
import { isPlatformServer } from '@angular/common';
import { catchError, filter, first, switchMap, tap } from 'rxjs/operators';
import { select, Store } from '@ngrx/store';
import { TransferState } from '@angular/platform-browser';
import * as navNodeActions from '../modules/global-header/actions/global-header/nav-nodes/nav-nodes.action';
import { State } from '../reducers';
import * as fromApp from '../reducers';

@Injectable()
export class NavNodesResolver implements Resolve<NavNodes> {
  constructor(private langService: LanguageService,
    private service: DataFetcher,
    private originService: OriginService,
    private transferState: TransferState,
    private store: Store<State>,
    private lang: LanguageService,
    @Inject(PLATFORM_ID) private platformId) {
  }

  resolve(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<NavNodes> {
    const lang = this.langService.getLanguage();
    const URL = apiUrls.BASE_URL(this.originService.getOrigin()) + apiUrls.GET_NAV_NODES();

    if (isPlatformServer(this.platformId)) {

      return this.service.fetch(URL, lang).pipe(
        tap(data => {
          this.store.dispatch(new navNodeActions.LoadNavNodesSuccess(data));
        }),
        catchError((error) => {
          return of(error);
        }),
        first()
      );

    } else {

      return this.store.pipe(
        select(fromApp.getNavNodes),
        tap(navNodes => {
          if (!navNodes.client) {
            this.store.dispatch(new navNodeActions.LoadNavNodes(lang));
          }
        }),
        filter(navNodes => navNodes.client),
        first()
      );

    }

  }

}
