import { HttpClient } from '@angular/common/http';
import { of } from 'rxjs/observable/of';
import { DataFetcher } from '@core/services/data-fetcher/data-fetcher.service';
import { TokenRefreshService } from '@core/services/token-refresh/token-refresh.service';

export const ui = null;
export const accessibility = null;

describe('Cart', () => {
  describe('Fetch data service, `fetch` method', () => {
    let mockService: any;
    let fakeHttpService: HttpClient;
    let dataService: DataFetcher;
    let fakeData: Object;
    let fakeResponseSuccess: Object;
    let $get: any;
    let mockTokenRefreshService: jasmine.SpyObj<TokenRefreshService>;

    beforeEach(() => {
      fakeData = { id: 1, name: 'fake' };
      fakeResponseSuccess = { status: 200 };
      fakeHttpService = {
        get: (_: any) => of([fakeData]),
        post: (url: string, body: any) => of([fakeResponseSuccess]),
        put: (url: string, body: any) => of([fakeResponseSuccess]),
        delete: (url: string, options: any) => of([fakeResponseSuccess])
      } as HttpClient;
      mockService = jasmine.createSpyObj('fakeHttpGetService', ['get', 'post', 'put', 'delete']);
      mockTokenRefreshService = jasmine.createSpyObj<TokenRefreshService>('TokenRefreshService', {'checkExpiry': { 'concatMap': (data) => of(data) }});
    });

    it('should have http `get` called', () => {
      mockService.get.and.returnValue(of([]));
      dataService = new DataFetcher(fakeHttpService, mockTokenRefreshService);
      dataService.fetch('', '', { 'req': 'get' });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `get` call with response data', () => {
      mockService.get.and.returnValue(of([fakeData]));
      dataService = new DataFetcher(fakeHttpService, mockTokenRefreshService);
      $get = dataService.fetch('', '', { 'req': 'get' });
      $get.subscribe(data => {
          data().subscribe(res => {
            expect(res).toEqual([fakeData]);
        });
      });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `post` called', () => {
      mockService.post.and.returnValue(of([]));
      dataService = new DataFetcher(fakeHttpService, mockTokenRefreshService);
      dataService.fetch('', '', { 'req': 'post' });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `post` call with response success', () => {
      mockService.post.and.returnValue(of([fakeResponseSuccess]));
      new DataFetcher(fakeHttpService, mockTokenRefreshService).fetch('', '', { 'req': 'post'}).subscribe(res => {
        res().subscribe(data => {
            expect(data).toEqual([fakeResponseSuccess]);
        });
      });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `put` called', () => {
      mockService.put.and.returnValue(of([]));
      dataService = new DataFetcher(fakeHttpService, mockTokenRefreshService);
      dataService.fetch('', '', { 'req': 'put' });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `put` call with response success', () => {
      mockService.put.and.returnValue(of([fakeResponseSuccess]));
      new DataFetcher(fakeHttpService, mockTokenRefreshService).fetch('', '', { 'req': 'put'}).subscribe(res => {
        res().subscribe(data => {
            expect(data).toEqual([fakeResponseSuccess]);
        });
      });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `delete` called', () => {
      mockService.delete.and.returnValue(of([]));
      dataService = new DataFetcher(fakeHttpService, mockTokenRefreshService);
      dataService.fetch('', '', { 'req': 'delete' });
      expect(dataService.fetch).toBeDefined();
    });

    it('should have http `delete` call with response success', () => {
      mockService.delete.and.returnValue(of([fakeResponseSuccess]));
      new DataFetcher(fakeHttpService, mockTokenRefreshService).fetch('', '', { 'req': 'delete'}).subscribe(res => {
        res().subscribe(data => {
            expect(data).toEqual([fakeResponseSuccess]);
        });
      });
      expect(dataService.fetch).toBeDefined();
    });
  });
});
