import { TranslateModule, TranslateLoader } from '@ngx-translate/core';
import { HttpClient } from '@angular/common/http';
import { Inject, NgModule, PLATFORM_ID, ModuleWithProviders } from '@angular/core';
import { CommonModule, LowerCasePipe } from '@angular/common';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { NgStringPipesModule } from 'angular-pipes';
import { CookieModule } from 'ngx-cookie';
import { ApiParamInfoService } from './services/data/api-param-info.service';
import { ScriptsService } from './services/scripts/scripts.service';
import { CookiesService } from './services/cookies/cookies.service';
import { UserInfoService } from './services/user-info/user-info.service';
import { TranslatesBrowserLoaderServiceShared } from './services/translation/translation-browser/translation-browser-loader.service';
import { NguCarouselModule } from '@ngu/carousel';
import { AngularClModule } from '@hdca/fe-angular-cl';
// modals
import { HDCAPageComponent } from './components/hdca-page/hdca-page.component';

// components
import { LanguageToggleComponent } from './components/micro-components/language-toggle/language-toggle.component';
import { VersionComponent } from './components/version/version.component';

import { ProductCarouselComponent } from './components/product-carousel/product-carousel.component';
import { TooltipComponent } from './components/tooltip/tooltip.component';
import { AlertComponent } from './components/alert-dialog/alert.component';
// containers
import { ManageSubscriptionsContainer } from './containers/manage-subs/manage-subscriptions.container';
import { TooltipDirective } from './directives/tooltip/tooltip.directive';
import { AppShellNoRenderDirective } from './directives/app-shell-no-render/app-shell-no-render.directive';
import { AppShellRenderDirective } from './directives/app-shell-render/app-shell-render.directive';
import { QtyInputMaxDirective } from './directives/qty-input-max/qty-input-max.directive';

// pipes
import { HighlightPipe } from './pipes/highlight/highlight.pipe';
import { OrderByPipe } from './pipes/order-by/order-by.pipe';
import { UnsafePipe } from './pipes/unsafe/unsafe.pipe';
import { DecodeHtmlStringPipe } from './pipes/decode-html/decode-html.pipe';
import { TransformUrlPipe } from './pipes/transform-url/transformUrl.pipe';
import { FormatPhonePipe } from './pipes/format-phone/format-phone.pipe';
// date picker
import { DatePickerComponent } from './components/date-picker/date-picker.component';
import { DateTimeService } from './services/date-time/date-time.service';

// language
import { LanguageService } from './services/router-language/route-language.service';
import { LanguageGuard } from './routes/language.guard';
import { StoreService } from './services/store/store.service';
import { TransferState } from '@angular/platform-browser';
import { NgxJsonLdModule } from '@ngx-lite/json-ld';
import { RouterModule } from '@angular/router';
import { DashCasePipe } from './pipes/dash-case/dash-case.pipe';

// SEO
import { MetaLink } from './services/meta-link/meta-link.service';

import { domain, production } from './tokens';

export function translateStaticLoaderShared(http: HttpClient, transferState: TransferState): TranslatesBrowserLoaderServiceShared {
  return new TranslatesBrowserLoaderServiceShared('/node-assets/i18n/shared/', '.json', transferState, http);
}

export interface IEnvironment {
  domain: string;
  appName: string;
  envName: string;
  production: boolean;
  versions: any;
}
@NgModule({
  imports: [
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    NgStringPipesModule,
    CookieModule.forChild(),
    TranslateModule.forChild({
      loader: {
        provide: TranslateLoader,
        useFactory: translateStaticLoaderShared,
        deps: [HttpClient, TransferState]
      }
    }),
    NguCarouselModule,
    NgxJsonLdModule,
    RouterModule,
    AngularClModule
  ],
  declarations: [
    AlertComponent,
    HDCAPageComponent,
    // containers
    ManageSubscriptionsContainer,
    VersionComponent,
    LanguageToggleComponent,
    ProductCarouselComponent,
    TooltipComponent,
    TooltipComponent,
    ProductCarouselComponent,
    // directives
    AppShellNoRenderDirective,
    AppShellRenderDirective,
    TooltipDirective,
    QtyInputMaxDirective,
    // pipes
    HighlightPipe,
    OrderByPipe,
    DatePickerComponent,
    DashCasePipe,
    UnsafePipe,
    DecodeHtmlStringPipe,
    TransformUrlPipe,
    FormatPhonePipe
  ],
  exports: [
    AlertComponent,
    CommonModule,
    FormsModule,
    ReactiveFormsModule,
    NgStringPipesModule,
    CookieModule,
    ManageSubscriptionsContainer,
    VersionComponent,
    LanguageToggleComponent,
    HDCAPageComponent,
    DatePickerComponent,
    TooltipComponent,
    TooltipComponent,
    ProductCarouselComponent,
    AppShellNoRenderDirective,
    AppShellRenderDirective,
    QtyInputMaxDirective,
    HighlightPipe,
    NguCarouselModule,
    TooltipDirective,
    FormatPhonePipe,
    HighlightPipe,
    OrderByPipe,
    UnsafePipe,
    DecodeHtmlStringPipe,
    TransformUrlPipe
  ],
  entryComponents: [
     ManageSubscriptionsContainer,
     AlertComponent,
     VersionComponent,
     LanguageToggleComponent,
     HDCAPageComponent,
     DatePickerComponent,
     TooltipComponent,
     TooltipComponent,
     ProductCarouselComponent
  ]
})
export class HDCASharedModule {
  constructor(private languageService: LanguageService,
    @Inject(PLATFORM_ID) platformId: any) {
    const lang = this.languageService.getLanguage();
    this.languageService.setLanguage(lang);
  }

  static forRoot(env: IEnvironment): ModuleWithProviders {
    return {
      ngModule: HDCASharedModule,
      providers: [
        LanguageService,
        StoreService,
        LanguageGuard,
        ApiParamInfoService,
        ScriptsService,
        DateTimeService,
        MetaLink,
        DashCasePipe,
        LowerCasePipe,
        UnsafePipe,
        DecodeHtmlStringPipe,
        TransformUrlPipe,
        CookiesService,
        UserInfoService,
        {
          provide: domain,
          useValue: env.domain
        },
        { provide: production,
          useValue: env.production }
      ]
    };
  }

}
