import { Injectable, OnDestroy } from '@angular/core';
import { CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, Router } from '@angular/router';
import { Observable } from 'rxjs/Observable';
import { LanguageService } from '../services/router-language/route-language.service';
import { LangConsts } from '../constants/lang.constants';
import { Subscription } from 'rxjs';

@Injectable()
export class LanguageGuard implements CanActivate, OnDestroy {
  langSub: Subscription;
  constructor(private languageService: LanguageService, private router: Router) {
  }

  ngOnDestroy() {
    this.langSub.unsubscribe();
  }

  canActivate(
    next: ActivatedRouteSnapshot,
    state: RouterStateSnapshot): Observable<boolean> | Promise<boolean> | boolean {
    let serverRouteLanguage = this.languageService.getLanguage(state.url);
    const routeLanguage = next.data.lang;

    if (!serverRouteLanguage) {
      // set default language
      const language = routeLanguage ? routeLanguage : LangConsts.DEFAULT_LANG;
      serverRouteLanguage = language;
    }
    this.languageService.setLanguage(serverRouteLanguage);
    this.langSub = this.languageService.translateURLAsync(state.url).subscribe((translatedRoutes) => {
      let URL = state.url;
      if (URL.substr(0, 1) === '/') {
        URL = URL.substr(1);
      }
      let translatedRoute = '/';
      URL.split('/').forEach(key => translatedRoute += translatedRoutes[key] + '/');
      if (translatedRoute === state.url || translatedRoute === state.url + '/') {
        return true;
      }
      this.router.navigate([translatedRoute]);
      return false;
    });
    return true;
  }
}
