import * as AdobeAnalyticsConstants from '../constants/adobe-analytics.constants';
import * as AdobeAnalyticsModels from '../models/adobe-analytics-data-layer-models.interface';
import { AdobeAnalyticsPayloadService } from './adobe-analytics-payload.service';
import { UserInfoService } from '@hdca/fe-angular-shared';
import { DateTimeService } from '@hdca/fe-angular-shared';
import { ProductUnlocalizedData } from '@app/modules/product/models/product-unlocalized-data.interface';
import { mockMultipleProductUnlocalizedData } from '@app/core/stubs/product/product-unlocalized-data.stub';
import { ProductLocalizedData } from '@app/modules/product/models/product-localized-data.interface';
import { mockProductLocalizedData } from '@app/core/stubs/product/product-localized-data.stub';
import { LanguageService } from '@hdca/fe-angular-shared';
import { userWithCartData } from '@app/core/stubs/cart/cart.stub';
import { mockProductFbtLocalizedData } from '@app/core/stubs/product/product-localized-fbt-data.stub';

const mockEventBlock: AdobeAnalyticsModels.EventBlock = {
  type: AdobeAnalyticsConstants.eventTypes.pageLoad,
  timestamp: 101,
  page: AdobeAnalyticsConstants.pageTypes.pip,
  language: AdobeAnalyticsConstants.languageTypes.en
};

const mockPageDetailDDO: AdobeAnalyticsModels.PageBlock = {
  pageName: 'Vivere 8 ft. Combo Steel Stand with Double Hammock',
  pageContentType: AdobeAnalyticsConstants.pageTypes.pip,
  sysEnv: AdobeAnalyticsConstants.systemEnvironments.desktop,
  pageSiteSection: AdobeAnalyticsConstants.pageTypes.pip,
  breadcrumbs: [{
    'title': 'Home',
    'url': '/en/home.html'
  }, {
    'title': '1000755816',
    'url': '/en/home/p.dewalt-dcd796d2-20v-max-xr-li-ion-cordless-brushless-compact-hammer-drill-kit.1000755816.html'
  }],
  'pageCategory': 'Home',
  'pageCategory1': '1000755816',
  'pageCategory2': '',
  'pageCategory3': '',
  'pageCategory4': '',
  'pageCategory5': '',
  'search' : {
    redirect: false,
    keyword: ''
  }
};

const mockCartBlock: AdobeAnalyticsModels.CartBlock = {
  'active': true,
  'id': '7aeac412-9c7e-4063-97cd-31c3e9a428ea',
  'item': [{
    'productId': '1000848230',
    'price': {
      'baseprice': 329,
      'discountPrice': 329,
      'voucherCode': 6,
      'discount': 1,
      'shippingMethod': undefined,
    },
    'quantity': 2,
    'itemTotal': 658,
    'deliveryDays': undefined,
  }],
  'attributes': {
    'totalUnits': 4,
    'shipping': {
      'mixed': ['BOSS', 'APPL', 'SHIPTOHOME', 'BOPIS', 'BODFS'],
    },
    'productcount': 1,
    'deliveryCost': 2,
  }
};

const mockPageOverviewDDO = {
  'productId': '',
  'attributes': {
      'eligible': 'BOPIS:INSO',
      'freeShipping': 1,
      'rating': 3,
      'reviews': 2,
      'questions': 5,
      'storeShippingAvailability': 'outOfStock',
      'regPrice': '$58.16',
      'wasPrice': '',
      'storeStockAvailability': 'inStock',
      'onlineStockAvailability': 'inStock',
      'localizedAssortment': 'Sold In-Store Only',
    },
    'superSku': {
      'eligible': true,
      'attributeCount': 2,
      'attributes': [{
        'activeValueId': 'vvc-cls_l3-blinds-shades-attribute_id_ColourFamily-yellow',
        'activeValueName': 'Yellow',
        'attributeId': 'vc-cls_l3-blinds-shades-attribute_id_ColourFamily',
        'attributeName': 'Color',
        'valueIds': ['vvc-cls_l3-blinds-shades-attribute_id_ColourFamily-green', 'vvc-cls_l3-blinds-shades-attribute_id_ColourFamily-yellow'],
        'valueNames': ['vc-cls_l3-blinds-shades-attribute_id_ColourFamily', 'vc-cls_l3-blinds-shades-attribute_id_ColourFamily'],
      }, {
        'activeValueId': 'vvc-cls_l3-blinds-shades-attribute_id_SizeFamily-8_9',
        'activeValueName': '8X9',
        'attributeId': 'vc-cls_l3-blinds-shades-attribute_id_SizeFamily',
        'attributeName': 'Size',
        'valueIds': ['vvc-cls_l3-blinds-shades-attribute_id_SizeFamily-5_7', 'vvc-cls_l3-blinds-shades-attribute_id_SizeFamily-8_9'],
        'valueNames': ['vc-cls_l3-blinds-shades-attribute_id_SizeFamily', 'vc-cls_l3-blinds-shades-attribute_id_SizeFamily'],
      }]
    },
    'fbt': {
      'item': [
        {'slotLocation': 1, 'productId': '1000755816', 'sellPrice': 180},
        {'slotLocation': 2, 'productId': '1000755818', 'sellPrice': 165},
        {'slotLocation': 3, 'productId': '1000755819', 'sellPrice': 165},
      ]
    }
  };

const mockPageLoadPayload: AdobeAnalyticsModels.PageAndModalViewPayload = {
  event: mockEventBlock,
  pageDetail: mockPageDetailDDO,
  productOverview: mockPageOverviewDDO,
  cart: mockCartBlock,
};

const getMockProductUnlocalizedData = (): ProductUnlocalizedData => {
  const _mockProductUnlocalizedData: ProductUnlocalizedData = mockMultipleProductUnlocalizedData['1000755816'];
  return { ..._mockProductUnlocalizedData };
};

const getMockProductLocalizedData = (): ProductLocalizedData => {
    const _mockProductLocalizedData: ProductLocalizedData = mockProductLocalizedData['1000755816'];
  return { ..._mockProductLocalizedData };
};

const mockProductCombineData = {
  productUnlocalizedData: getMockProductUnlocalizedData(),
  productUnlocalizedErrorData: null,
  productLocalizedData: getMockProductLocalizedData(),
  productLocalizedErrorData: null
};

const getMockProductFbtData = () => {
  const _mockProductFbtLocalizedData = mockProductFbtLocalizedData['1000755816'];
  return { ..._mockProductFbtLocalizedData };
};

const mockFbtData: any = {
  fbt: getMockProductFbtData(),
};

const mockRedirectData: any = {
  redirect: false,
  keyword: ''
};

const mockCartData: any = {
  ...userWithCartData,
  loading: false,
  error: {
    errorObj: undefined
  }
};

describe('Adobe Analytics Payload Service', () => {
  let userInfoServiceSpy: jasmine.SpyObj<UserInfoService>;
  let dateTimeServiceSpy: jasmine.SpyObj<DateTimeService>;
  let service: AdobeAnalyticsPayloadService;
  let languageServiceMock: jasmine.SpyObj<LanguageService>;

  const originalDateNow = Date.now;
  beforeEach(() => {
    userInfoServiceSpy = jasmine.createSpyObj<UserInfoService>('UserInfoService', ['getUserAccountData']);
    dateTimeServiceSpy = jasmine.createSpyObj<DateTimeService>('DateTimeService', ['format', 'getFormattedDate']);
    languageServiceMock = jasmine.createSpyObj('LanguageService', ['getLanguage']);
    service = new AdobeAnalyticsPayloadService(userInfoServiceSpy, languageServiceMock);
    mockPageLoadPayload.productOverview = service['returnProduct'](mockProductCombineData.productUnlocalizedData, mockProductCombineData.productLocalizedData, mockFbtData);
    Date.now = () => 101;
  });

  afterEach(() => {
    Date.now = originalDateNow;
  });

  it('should have created', () => {
    expect(service).toBeDefined();
  });

  it('should be called', () => {
    spyOn(service, 'formPageLoadPayload');
    const dataWrapper = {
      'cartData': mockCartData,
      'redirect': mockRedirectData,
      'productUnlocalized': mockProductCombineData.productUnlocalizedData,
      'productLocalized': mockProductCombineData.productLocalizedData,
      'fbtData': mockFbtData
    };
    const result = service.formPageLoadPayload(mockEventBlock, dataWrapper, null);
    expect(service.formPageLoadPayload).toHaveBeenCalled();
  });

  it('should returned true', () => {
    const dataWrapper = {
      'cartData': mockCartData,
      'redirect': mockRedirectData,
      'productUnlocalized': mockProductCombineData.productUnlocalizedData,
      'productLocalized': mockProductCombineData.productLocalizedData,
      'fbtData': mockFbtData
    };
    const result = service.formPageLoadPayload(mockEventBlock, dataWrapper, null);
    expect(result).toBeTruthy();
  });

  it('should returned proper result', () => {
    const dataWrapper = {
      'cartData': mockCartData,
      'redirect': mockRedirectData,
      'productUnlocalized': mockProductCombineData.productUnlocalizedData,
      'productLocalized': mockProductCombineData.productLocalizedData,
      'fbtData': mockFbtData
    };

    const result = service.formPageLoadPayload(mockEventBlock, dataWrapper, null);
    expect(result).toEqual(mockPageLoadPayload);
  });

});
