import * as constants from '../constants/adobe-analytics.constants';
import { Injectable } from '@angular/core';

/**
 *  Runtime Payload Verification
 */
@Injectable()
export class PayloadVerificationService {
    constructor() {}

    isElementClickPayload(payload) {
      return payload.elementType in constants.clickElementTypes
        && payload.elementValue in constants.clickElementValues;
    }

    isPageAndModalViewPayload(payload) {
      return payload.pageDetail
        && typeof payload.pageDetail.pageName === 'string'
        && payload.pageDetail.sysEnv in constants.systemEnvironments
        && payload.pageDetail.pageContentType in constants.pageTypes;
    }

    isSiteErrorPayload(payload) {
      return payload.errorType in constants.siteErrorTypes
        && typeof payload.userInformed === 'boolean'
        && typeof payload.errorCode === 'string'
        && typeof payload.errorValue === 'string';
    }

    isAddToCartPayload(payload) {
      return Array.isArray(payload) && payload.every(item =>
        !!item.product
        && item.product.code
        && (typeof item.itemQuantity === 'string' || typeof item.itemQuantity === 'number')
        && typeof item.isCartSwitch === 'boolean'
        && item.shippingMethod in constants.shippingMethodTypes
        && item.addToCartLocation in constants.productInteractionLocationTypes
        && typeof item.slotLocation === 'number');
    }

    isSuperSkuSwitchPayload(payload) {
      return payload.productId && payload.attributeId && payload.attributeName
        && payload.targetProductId && payload.targetValueId && payload.targetValueName;
    }

    isCarouselItemClickPayload(payload) {
      return payload.productId && payload.contextProductId && payload.slotLocation
          && payload.productInteractionLocation && payload.productInteractionPageType && payload.productInteractionLocationTitle;
    }

    isWhitelistPayload(payload) {
      return !!payload.global && !!payload.global.analytics && Array.isArray(payload.global.analytics);
    }

}
